@@ -76,6 +76,8 @@ list_loadconf (char *file, GSList ** list, char *defaultconf)
 	fh = open (filebuf, O_RDONLY | OFLAGS);
 	if (fh == -1)
 	{
+		if (!defaultconf)
+			return;
 		fh = open (filebuf, O_TRUNC | O_WRONLY | O_CREAT | OFLAGS, 0600);
 		if (fh != -1)
 		{
@@ -154,7 +156,7 @@ list_delentry (GSList ** list, char *name)
 }
 
 char *
-cfg_get_str (char *cfg, char *var, char *dest)
+cfg_get_str (char *cfg, char *var, char *dest, int dest_len)
 {
 	while (1)
 	{
@@ -175,7 +177,7 @@ cfg_get_str (char *cfg, char *var, char *dest)
 				cfg++;
 			t = *cfg;
 			*cfg = 0;
-			strcpy (dest, value);
+			safe_strcpy (dest, value, dest_len);
 			*cfg = t;
 			return cfg;
 		}
@@ -219,7 +221,7 @@ cfg_get_int_with_result (char *cfg, char *var, int *result)
 {
 	char str[128];
 
-	if (!cfg_get_str (cfg, var, str))
+	if (!cfg_get_str (cfg, var, str, sizeof (str)))
 	{
 		*result = 0;
 		return 0;
@@ -234,7 +236,7 @@ cfg_get_int (char *cfg, char *var)
 {
 	char str[128];
 
-	if (!cfg_get_str (cfg, var, str))
+	if (!cfg_get_str (cfg, var, str, sizeof (str)))
 		return 0;
 
 	return atoi (str);
@@ -371,9 +373,9 @@ const struct prefs vars[] = {
 	{"completion_old", P_OFFINT (old_nickcompletion), TYPE_BOOL},
 	{"completion_suffix", P_OFFSET (nick_suffix), TYPE_STR},
 
-	{"dcc_auto_chat", P_OFFINT (autodccchat), TYPE_BOOL},
+	{"dcc_auto_chat", P_OFFINT (autodccchat), TYPE_INT},
 	{"dcc_auto_resume", P_OFFINT (autoresume), TYPE_BOOL},
-	{"dcc_auto_send", P_OFFINT (autodccsend), TYPE_BOOL},
+	{"dcc_auto_send", P_OFFINT (autodccsend), TYPE_INT},
 	{"dcc_blocksize", P_OFFINT (dcc_blocksize), TYPE_INT},
 	{"dcc_completed_dir", P_OFFSET (dcc_completed_dir), TYPE_STR},
 	{"dcc_dir", P_OFFSET (dccdir), TYPE_STR},
@@ -506,6 +508,7 @@ const struct prefs vars[] = {
 	{"tab_notices", P_OFFINT (notices_tabs), TYPE_BOOL},
 	{"tab_position", P_OFFINT (tabs_position), TYPE_INT},
 	{"tab_server", P_OFFINT (use_server_tab), TYPE_BOOL},
+	{"tab_sort", P_OFFINT (tab_sort), TYPE_BOOL},
 	{"tab_trunc", P_OFFINT (truncchans), TYPE_INT},
 	{"tab_utils", P_OFFINT (windows_as_tabs), TYPE_BOOL},
 
@@ -528,11 +531,30 @@ const struct prefs vars[] = {
 	{0, 0, 0},
 };
 
+static char *
+convert_with_fallback (const char *str, const char *fallback)
+{
+	char *utf;
+
+	utf = g_locale_to_utf8 (str, -1, 0, 0, 0);
+	if (!utf)
+	{
+		/* this can happen if CHARSET envvar is set wrong */
+		/* maybe it's already utf8 (breakage!) */
+		if (!g_utf8_validate (str, -1, NULL))
+			utf = g_strdup (fallback);
+		else
+			utf = g_strdup (str);
+	}
+
+	return utf;
+}
+
 void
 load_config (void)
 {
 	struct stat st;
-	char *cfg;
+	char *cfg, *sp;
 	const char *username, *realname;
 	int res, val, i, fh;
 
@@ -545,6 +567,9 @@ load_config (void)
 	if ((realname && realname[0] == 0) || !realname)
 		realname = username;
 
+	username = convert_with_fallback (username, "username");
+	realname = convert_with_fallback (realname, "realname");
+
 	memset (&prefs, 0, sizeof (struct xchatprefs));
 
 	/* put in default values, anything left out is automatically zero */
@@ -601,7 +626,6 @@ load_config (void)
 	prefs.autoopendccsendwindow = 1;
 	prefs.autoopendccchatwindow = 1;
 	prefs.userhost = 1;
-	prefs.perc_color = 1;
 	prefs.dcc_send_fillspaces = 1;
 	prefs.mainwindow_save = 1;
 	prefs.bantype = 2;
@@ -638,6 +662,9 @@ load_config (void)
 	strcpy (prefs.soundcmd, "esdplay");
 	strcpy (prefs.dnsprogram, "host");
 
+	g_free ((char *)username);
+	g_free ((char *)realname);
+
 	fh = open (default_file (), OFLAGS | O_RDONLY);
 	if (fh != -1)
 	{
@@ -654,7 +681,8 @@ load_config (void)
 			switch (vars[i].type)
 			{
 			case TYPE_STR:
-				cfg_get_str (cfg, vars[i].name, (char *) &prefs + vars[i].offset);
+				cfg_get_str (cfg, vars[i].name, (char *) &prefs + vars[i].offset,
+								 vars[i].len);
 				break;
 			case TYPE_BOOL:
 			case TYPE_INT:
@@ -687,6 +715,10 @@ load_config (void)
 		prefs.mainwindow_height = 138;
 	if (prefs.mainwindow_width < 106)
 		prefs.mainwindow_width = 106;
+
+	sp = strchr (prefs.username, ' ');
+	if (sp)
+		sp[0] = 0;	/* spaces in username would break the login */
 }
 
 int
@@ -762,7 +794,7 @@ static void
 set_showval (session *sess, const struct prefs *var, char *tbuf)
 {
 	int len, dots, j;
-	static char *offon[] = { "OFF", "ON" };
+	static const char *offon[] = { "OFF", "ON" };
 
 	len = strlen (var->name);
 	memcpy (tbuf, var->name, len);
@@ -859,8 +891,9 @@ cmd_set (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			case TYPE_STR:
 				if (*val)
 				{
-					strcpy ((char *) &prefs + vars[i].offset, val);
-					PrintTextf (sess, "%s set to: %s\n", var, val);
+					strncpy ((char *) &prefs + vars[i].offset, val, vars[i].len);
+					((char *) &prefs)[vars[i].offset + vars[i].len - 1] = 0;
+					PrintTextf (sess, "%s set to: %s\n", var, (char *) &prefs + vars[i].offset);
 				} else
 				{
 					set_showval (sess, &vars[i], tbuf);
