@@ -35,8 +35,13 @@
 
 #define WANTSOCKET
 #define WANTARPA
+#define WANTDNS
 #include "inet.h"
 
+#ifdef WIN32
+#include <windows.h>
+#endif
+
 #include "xchat.h"
 #include "util.h"
 #include "fe.h"
@@ -100,21 +105,6 @@ dcc_unthrottle (struct DCC *dcc)
 		dcc_send_data (NULL, 0, dcc);
 }
 
-#ifdef WIN32
-#include <windows.h>
-
-/* g_get_current_time is giving bad CPS results, let's try this instead */
-static void
-g_get_current_time_win32 (GTimeVal *tv)
-{
-	SYSTEMTIME current;
-
-	GetLocalTime (&current);
-	tv->tv_sec = current.wSecond;
-	tv->tv_usec = current.wMilliseconds * 1000;
-}
-#endif
-
 static void
 dcc_calc_cps (struct DCC *dcc)
 {
@@ -125,11 +115,7 @@ dcc_calc_cps (struct DCC *dcc)
 	int *cpssum, glob_limit;
 	unsigned int pos, posdiff;
 
-#ifdef WIN32
-	g_get_current_time_win32 (&now);
-#else
 	g_get_current_time (&now);
-#endif
 
 	/* the pos we use for sends is an average
 		between pos and ack */
@@ -149,7 +135,7 @@ dcc_calc_cps (struct DCC *dcc)
 		glob_limit = prefs.dcc_global_max_get_cps;
 	}
 
-	if (!dcc->firstcpstv.tv_sec)
+	if (!dcc->firstcpstv.tv_sec && !dcc->firstcpstv.tv_usec)
 		dcc->firstcpstv = now;
 	else
 	{
@@ -653,7 +639,7 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 				do
 				{
 					n++;
-					sprintf (buf, "%s.%d", dcc->destfile_fs, n);
+					snprintf (buf, sizeof (buf), "%s.%d", dcc->destfile_fs, n);
 				}
 				while (access (buf, F_OK) == 0);
 
@@ -674,8 +660,9 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	}
 	if (dcc->fp == -1)
 	{
+		/* the last executed function is open(), errno should be valid */
 		EMIT_SIGNAL (XP_TE_DCCFILEERR, dcc->serv->front_session, dcc->destfile,
-						 NULL, NULL, NULL, 0);
+						 errorstring (errno), NULL, NULL, 0);
 		dcc_close (dcc, STAT_FAILED, FALSE);
 		return TRUE;
 	}
@@ -800,6 +787,7 @@ dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dc
 		break;
 	case TYPE_SEND:
 		/* passive send */
+		dcc->fastsend = prefs.fastdccsend;
 		if (dcc->fastsend)
 			dcc->wiotag = fe_input_add (dcc->sok, FIA_WRITE, dcc_send_data, dcc);
 		dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_read_ack, dcc);
@@ -972,13 +960,7 @@ dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	if (dcc->ackoffset)
 		dcc->ack += dcc->resumable;
 
-	if (!dcc->fastsend)
-	{
-		if (dcc->ack < dcc->pos)
-			return TRUE;
-		dcc_send_data (NULL, 0, (gpointer)dcc);
-	}
-
+	/* DCC complete check */
 	if (dcc->pos >= dcc->size && dcc->ack >= dcc->size)
 	{
 		dcc_calc_average_cps (dcc);
@@ -987,6 +969,10 @@ dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 		EMIT_SIGNAL (XP_TE_DCCSENDCOMP, dcc->serv->front_session,
 						 file_part (dcc->file), dcc->nick, buf, NULL, 0);
 	}
+	else if ((!dcc->fastsend) && (dcc->ack >= dcc->pos))
+	{
+		dcc_send_data (NULL, 0, (gpointer)dcc);
+	}
 
 	return TRUE;
 }
@@ -1060,6 +1046,7 @@ dcc_listen_init (struct DCC *dcc, session *sess)
 {
 	unsigned long my_addr;
 	struct sockaddr_in SAddr;
+	struct hostent *dns_query;
 	int i, bindretval = -1;
 	socklen_t len;
 
@@ -1097,11 +1084,14 @@ dcc_listen_init (struct DCC *dcc, session *sess)
 			/*printf("Trying to bind against port: %d\n",ntohs(SAddr.sin_port));*/
 			bindretval = bind (dcc->sok, (struct sockaddr *) &SAddr, sizeof (SAddr));
 		}
-	}
 
-	/*if we didnt bind yet, try a random port*/
-	if (bindretval == -1)
+		/* with a small port range, reUseAddr is needed */
+		len = 1;
+		setsockopt (dcc->sok, SOL_SOCKET, SO_REUSEADDR, (char *) &len, sizeof (len));
+
+	} else
 	{
+		/* try random port */
 		SAddr.sin_port = 0;
 		bindretval = bind (dcc->sok, (struct sockaddr *) &SAddr, sizeof (SAddr));
 	}
@@ -1119,14 +1109,30 @@ dcc_listen_init (struct DCC *dcc, session *sess)
 	dcc->port = ntohs (SAddr.sin_port);
 
 	/*if we have a dcc_ip, we use that, so the remote client can connect*/
-	/*we would tell the client to connect to our LAN ip otherwise*/
+	/*else we try to take an address from dcc_ip_str*/
+	/*if something goes wrong we tell the client to connect to our LAN ip*/
+
+	dcc->addr = 0;
+
 	if (prefs.ip_from_server != 0 && prefs.dcc_ip != 0)
 		dcc->addr = prefs.dcc_ip;
 	else if (prefs.dcc_ip_str[0])
-		dcc->addr = inet_addr ((const char *) prefs.dcc_ip_str);
-	else
-	/*else we use the one we bound to*/
-		dcc->addr = my_addr;
+	{
+	   dns_query = gethostbyname ((const char *) prefs.dcc_ip_str);
+
+	   if (dns_query != NULL &&
+	       dns_query->h_length == 4 &&
+	       dns_query->h_addr_list[0] != NULL)
+		  {
+		    /*we're offered at least one IPv4 address: we take the first*/
+
+		    dcc->addr = (unsigned long) *((guint32*) dns_query->h_addr_list[0]);
+		  }
+	}
+
+	/*if nothing else worked we use the address we bound to*/
+	if (dcc->addr == 0)
+	   dcc->addr = my_addr;
 
 	dcc->addr = ntohl (dcc->addr);
 
@@ -1165,6 +1171,7 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 	{
 		char path[256];
 		char wild[256];
+		char *path_fs;	/* local filesystem encoding */
 
 		safe_strcpy (wild, file_part (file), sizeof (wild));
 		path_part (file, path, sizeof (path));
@@ -1177,9 +1184,15 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 
 		free (file);
 
-		recursive = TRUE;
-		for_files (path, wild, dcc_send_wild);
-		recursive = FALSE;
+		/* for_files() will use opendir, so we need local FS encoding */
+		path_fs = g_filename_from_utf8 (path, -1, 0, 0, 0);
+		if (path_fs)
+		{
+			recursive = TRUE;
+			for_files (path_fs, wild, dcc_send_wild);
+			recursive = FALSE;
+			g_free (path_fs);
+		}
 
 		return;
 	}
@@ -1362,9 +1375,16 @@ dcc_get (struct DCC *dcc)
 	case STAT_QUEUED:
 		if (dcc->type != TYPE_CHATSEND)
 		{
-			dcc->resumable = 0;
-			dcc->pos = 0;
-			dcc_connect (dcc);
+			if (dcc->type == TYPE_RECV && prefs.autoresume && dcc->resumable)
+			{
+				dcc_resume (dcc);
+			}
+			else
+			{
+				dcc->resumable = 0;
+				dcc->pos = 0;
+				dcc_connect (dcc);
+			}
 		}
 		break;
 	case STAT_DONE:
@@ -1482,6 +1502,62 @@ dcc_malformed (struct session *sess, char *nick, char *data)
 	EMIT_SIGNAL (XP_TE_MALFORMED, sess, nick, data, NULL, NULL, 0);
 }
 
+static int
+is_resumable (struct DCC *dcc)
+{
+	dcc->resumable = 0;
+
+	/* Check the file size */
+	if (access (dcc->destfile_fs, W_OK) == 0)
+	{
+		struct stat st;
+
+		if (stat (dcc->destfile_fs, &st) != -1)
+		{
+			if (st.st_size < dcc->size)
+			{
+				dcc->resumable = st.st_size;
+				dcc->pos = st.st_size;
+			}
+			else
+				dcc->resume_error = 2;
+		} else
+		{
+			dcc->resume_errno = errno;
+			dcc->resume_error = 1;
+		}
+	} else
+	{
+		dcc->resume_errno = errno;
+		dcc->resume_error = 1;
+	}
+
+	/* Now verify that this DCC is not already in progress from someone else */
+
+	if (dcc->resumable)
+	{
+		GSList *list = dcc_list;
+		struct DCC *d;
+		while (list)
+		{
+			d = list->data;
+			if (d->type == TYPE_RECV && d->dccstat != STAT_ABORTED &&
+				 d->dccstat != STAT_DONE && d->dccstat != STAT_FAILED)
+			{
+				if (d != dcc && strcmp (d->destfile, dcc->destfile) == 0)
+				{
+					dcc->resumable = 0;
+					dcc->pos = 0;
+					break;
+				}
+			}
+			list = list->next;
+		}
+	}
+
+	return dcc->resumable;
+}
+
 int
 dcc_resume (struct DCC *dcc)
 {
@@ -1505,6 +1581,34 @@ dcc_resume (struct DCC *dcc)
 	return 0;
 }
 
+static void
+dcc_confirm_send (void *ud)
+{
+	struct DCC *dcc = (struct DCC *) ud;
+	dcc_get (dcc);
+}
+
+static void
+dcc_deny_send (void *ud)
+{
+	struct DCC *dcc = (struct DCC *) ud;
+	dcc_abort (dcc->serv->front_session, dcc);
+}
+
+static void
+dcc_confirm_chat (void *ud)
+{
+	struct DCC *dcc = (struct DCC *) ud;
+	dcc_connect (dcc);
+}
+
+static void
+dcc_deny_chat (void *ud)
+{
+	struct DCC *dcc = (struct DCC *) ud;
+	dcc_abort (dcc->serv->front_session, dcc);
+}
+
 void
 handle_dcc (struct session *sess, char *nick, char *word[],
 				char *word_eol[])
@@ -1559,8 +1663,14 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 			} else
 				fe_dcc_add (dcc);
 
-			if (prefs.autodccchat)
+			if (prefs.autodccchat == 1)
 				dcc_connect (dcc);
+			else if (prefs.autodccchat == 2)
+			{
+				char buff[128];
+				snprintf (buff, sizeof (buff), "%s is offering DCC Chat.  Do you want to accept?", nick);
+				fe_confirm (buff, dcc_confirm_chat, dcc_deny_chat, dcc);
+			}
 		}
 		return;
 	}
@@ -1659,17 +1769,21 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 			 * sending file.
 			 */
 			dcc = find_dcc_from_id (pasvid, TYPE_SEND);
-			dcc->addr = addr;
-			dcc->port = port;
-			dcc_connect(dcc);
+			if (dcc)
+			{
+				dcc->addr = addr;
+				dcc->port = port;
+				dcc_connect (dcc);
+			} else
+			{
+				dcc_malformed (sess, nick, word_eol[4] + 2);
+			}
 			return;
 		}
 
 		dcc = new_dcc ();
 		if (dcc)
 		{
-			struct stat st;
-
 			dcc->file = strdup (file);
 
 			dcc->destfile = g_malloc (strlen (prefs.dccdir) + strlen (nick) +
@@ -1700,26 +1814,7 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 			dcc->destfile_fs = g_filename_from_utf8 (dcc->destfile, -1, 0, 0, 0);
 
 			dcc->resumable = 0;
-			if (access (dcc->destfile_fs, W_OK) == 0)
-			{
-				if (stat (dcc->destfile_fs, &st) != -1)
-				{
-					if (st.st_size < size)
-						dcc->resumable = st.st_size;
-					else
-						dcc->resume_error = 2;
-				} else
-				{
-					dcc->resume_errno = errno;
-					dcc->resume_error = 1;
-				}
-			} else
-			{
-				dcc->resume_errno = errno;
-				dcc->resume_error = 1;
-			}
-
-			dcc->pos = dcc->resumable;
+			dcc->pos = 0;
 			dcc->serv = sess->server;
 			dcc->type = TYPE_RECV;
 			dcc->dccstat = STAT_QUEUED;
@@ -1729,33 +1824,20 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 			dcc->size = size;
 			dcc->nick = strdup (nick);
 			dcc->maxcps = prefs.dcc_max_get_cps;
-			if (prefs.autodccsend)
+
+			is_resumable (dcc);
+
+			/* autodccsend is really autodccrecv.. right? */
+
+			if (prefs.autodccsend == 1)
 			{
-				if (prefs.autoresume && dcc->resumable)
-				{
-					/* don't resume the same file from two people! */
-					GSList *list = dcc_list;
-					struct DCC *d;
-					while (list)
-					{
-						d = list->data;
-						if (d->type == TYPE_RECV && d->dccstat != STAT_ABORTED &&
-							 d->dccstat != STAT_DONE && d->dccstat != STAT_FAILED)
-						{
-							if (d != dcc && strcmp (d->destfile, dcc->destfile) == 0)
-								goto dontresume;
-						}
-						list = list->next;
-					}
-					dcc_resume (dcc);
-				} else
-				{
-dontresume:
-					/*dcc->resume_error = 3;*/
-					dcc->resumable = 0;
-					dcc->pos = 0;
-					dcc_connect (dcc);
-				}
+				dcc_get (dcc);
+			}
+			else if (prefs.autodccsend == 2)
+			{
+				char buff[128];
+				snprintf (buff, sizeof (buff), "%s is offering \"%s\" via DCC.  Do you want to accept the transfer?", nick, file);
+				fe_confirm (buff, dcc_confirm_send, dcc_deny_send, dcc);
 			}
 			if (prefs.autoopendccrecvwindow)
 			{
