@@ -1,6 +1,9 @@
 #include "userlist.h"
 #include "dcc.h"
 
+#ifndef XCHAT_FE_H
+#define XCHAT_FE_H
+
 int fe_args (int argc, char *argv[]);
 void fe_init (void);
 void fe_main (void);
@@ -21,6 +24,7 @@ void fe_input_remove (int tag);
 void fe_idle_add (void *func, void *data);
 void fe_set_topic (struct session *sess, char *topic);
 void fe_set_hilight (struct session *sess);
+void fe_set_tab_color (struct session *sess, int col, int flash);
 void fe_update_mode_buttons (struct session *sess, char mode, char sign);
 void fe_update_channel_key (struct session *sess);
 void fe_update_channel_limit (struct session *sess);
@@ -70,4 +74,8 @@ void fe_set_away (server *serv);
 void fe_serverlist_open (session *sess);
 void fe_get_str (char *prompt, char *def, void *callback, void *ud);
 void fe_get_int (char *prompt, int def, void *callback, void *ud);
-void fe_ctrl_gui (session *sess, int action);
+void fe_ctrl_gui (session *sess, int action, int arg);
+int fe_gui_info (session *sess, int info_type);
+void fe_confirm (const char *message, void (*yesproc)(void *), void (*noproc)(void *), void *ud);
+
+#endif
