@@ -63,7 +63,8 @@ color_of (char *name)
 void
 clear_channel (session *sess)
 {
-	strcpy (sess->waitchannel, sess->channel);
+	if (sess->channel[0])
+		strcpy (sess->waitchannel, sess->channel);
 	sess->channel[0] = 0;
 	sess->doing_who = FALSE;
 	sess->done_away_check = FALSE;
@@ -169,7 +170,11 @@ inbound_privmsg (server *serv, char *from, char *ip, char *text)
 
 	sess = find_session_from_nick (from, serv);
 	if (!sess)
+	{
 		sess = serv->front_session;
+		EMIT_SIGNAL (XP_TE_PRIVMSG, sess, from, text, NULL, NULL, 0);
+		return;
+	}
 
 	if (sess->type == SESS_DIALOG)
 		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, NULL, NULL, 0);
@@ -220,7 +225,10 @@ is_hilight (char *text, session *sess, server *serv)
 	if ((SearchNick (text, serv->nick)) || SearchNick (text, prefs.bluestring))
 	{
 		if (sess != current_tab)
+		{
+			sess->nick_said = TRUE;
 			fe_set_hilight (sess);
+		}
 		return 1;
 	}
 	return 0;
@@ -653,7 +661,7 @@ inbound_topictime (server *serv, char *chan, char *nick, time_t stamp)
 	if (!sess)
 		sess = serv->server_session;
 
-	tim[19] = 0;	/* get rid of the \n */
+	tim[24] = 0;	/* get rid of the \n */
 	EMIT_SIGNAL (XP_TE_TOPICDATE, sess, chan, nick, tim, NULL, 0);
 }
 
@@ -1148,7 +1156,7 @@ inbound_login_end (session *sess, char *text)
 
 	if (!serv->end_of_motd)
 	{
-		if (prefs.ip_from_server)
+		if (prefs.ip_from_server && serv->use_who)
 		{
 			serv->skip_next_who = TRUE;
 			serv->p_get_ip (serv, serv->nick);	/* sends WHO mynick */
