@@ -499,7 +499,7 @@ cmd_unban (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		{
 			if (i == 2)
 				return FALSE;
-			send_channel_modes (sess, tbuf, word, 2, i, '-', 'b');
+			send_channel_modes (sess, tbuf, word, 2, i, '-', 'b', 0);
 			return TRUE;
 		}
 		i++;
@@ -835,7 +835,7 @@ cmd_dehop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		{
 			if (i == 2)
 				return FALSE;
-			send_channel_modes (sess, tbuf, word, 2, i, '-', 'h');
+			send_channel_modes (sess, tbuf, word, 2, i, '-', 'h', 0);
 			return TRUE;
 		}
 		i++;
@@ -853,7 +853,7 @@ cmd_deop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		{
 			if (i == 2)
 				return FALSE;
-			send_channel_modes (sess, tbuf, word, 2, i, '-', 'o');
+			send_channel_modes (sess, tbuf, word, 2, i, '-', 'o', 0);
 			return TRUE;
 		}
 		i++;
@@ -889,7 +889,7 @@ cmd_mdehop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	data.nicks = nicks;
 	data.i = 0;
 	tree_foreach (sess->usertree, (tree_traverse_func *)mdehop_cb, &data);
-	send_channel_modes (sess, tbuf, nicks, 0, data.i, '-', 'h');
+	send_channel_modes (sess, tbuf, nicks, 0, data.i, '-', 'h', 0);
 	free (nicks);
 
 	return TRUE;
@@ -915,7 +915,7 @@ cmd_mdeop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	data.nicks = nicks;
 	data.i = 0;
 	tree_foreach (sess->usertree, (tree_traverse_func *)mdeop_cb, &data);
-	send_channel_modes (sess, tbuf, nicks, 0, data.i, '-', 'o');
+	send_channel_modes (sess, tbuf, nicks, 0, data.i, '-', 'o', 0);
 	free (nicks);
 
 	return TRUE;
@@ -961,7 +961,7 @@ cmd_devoice (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		{
 			if (i == 2)
 				return FALSE;
-			send_channel_modes (sess, tbuf, word, 2, i, '-', 'v');
+			send_channel_modes (sess, tbuf, word, 2, i, '-', 'v', 0);
 			return TRUE;
 		}
 		i++;
@@ -1884,8 +1884,12 @@ cmd_load (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	len = strlen (word[2]);
 #ifdef WIN32
 	if (len > 4 && strcasecmp (".dll", word[2] + len - 4) == 0)
+#else
+#if defined(__hpux)
+	if (len > 3 && strcasecmp (".sl", word[2] + len - 3) == 0)
 #else
 	if (len > 3 && strcasecmp (".so", word[2] + len - 3) == 0)
+#endif
 #endif
 	{
 		arg = NULL;
@@ -1966,7 +1970,7 @@ cmd_mop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	data.nicks = nicks;
 	data.i = 0;
 	tree_foreach (sess->usertree, (tree_traverse_func *)mop_cb, &data);
-	send_channel_modes (sess, tbuf, nicks, 0, data.i, '+', 'o');
+	send_channel_modes (sess, tbuf, nicks, 0, data.i, '+', 'o', 0);
 
 	free (nicks);
 
@@ -2118,7 +2122,7 @@ cmd_op (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		{
 			if (i == 2)
 				return FALSE;
-			send_channel_modes (sess, tbuf, word, 2, i, '+', 'o');
+			send_channel_modes (sess, tbuf, word, 2, i, '+', 'o', 0);
 			return TRUE;
 		}
 		i++;
@@ -2289,13 +2293,6 @@ cmd_server (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	}
 #endif
 
-	/* connect by Network name */
-	if (word[3][0] == 0)
-	{
-		if (servlist_connect_by_netname (sess, word[2]))
-			return TRUE;
-	}
-
 	server_name = word[2 + offset];
 	port = word[3 + offset];
 	pass = word[4 + offset];
@@ -2303,6 +2300,10 @@ cmd_server (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	if (!(*server_name))
 		return FALSE;
 
+	/* dont clear it for /servchan */
+	if (strncasecmp (word_eol[1], "SERVCHAN ", 9))
+		sess->willjoinchannel[0] = 0;
+
 #ifdef USE_OPENSSL
 	if (strncasecmp ("ircs://", server_name, 7) == 0)
 	{
@@ -2312,10 +2313,6 @@ cmd_server (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	}
 #endif
 
-	/* dont clear it for /servchan */
-	if (strncasecmp (word_eol[1], "SERVCHAN ", 9))
-		sess->willjoinchannel[0] = 0;
-
 	if (strncasecmp ("irc://", server_name, 6) == 0)
 	{
 		server_name += 6;
@@ -2366,6 +2363,10 @@ urlserv:
 	serv->accept_invalid_cert = TRUE;
 #endif
 
+	/* try to connect by Network name */
+	if (servlist_connect_by_netname (sess, server_name))
+		return TRUE;
+
 	if (*port)
 	{
 		serv->connect (serv, server_name, atoi (port), FALSE);
@@ -2434,8 +2435,12 @@ cmd_unload (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	len = strlen (word[2]);
 #ifdef WIN32
 	if (len > 4 && strcasecmp (word[2] + len - 4, ".dll") == 0)
+#else
+#if defined(__hpux)
+	if (len > 3 && strcasecmp (word[2] + len - 3, ".sl") == 0)
 #else
 	if (len > 3 && strcasecmp (word[2] + len - 3, ".so") == 0)
+#endif
 #endif
 		by_file = TRUE;
 
@@ -2564,7 +2569,7 @@ cmd_hop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		{
 			if (i == 2)
 				return FALSE;
-			send_channel_modes (sess, tbuf, word, 2, i, '+', 'h');
+			send_channel_modes (sess, tbuf, word, 2, i, '+', 'h', 0);
 			return TRUE;
 		}
 		i++;
@@ -2582,7 +2587,7 @@ cmd_voice (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		{
 			if (i == 2)
 				return FALSE;
-			send_channel_modes (sess, tbuf, word, 2, i, '+', 'v');
+			send_channel_modes (sess, tbuf, word, 2, i, '+', 'v', 0);
 			return TRUE;
 		}
 		i++;
@@ -3077,7 +3082,7 @@ perform_nick_completion (struct session *sess, char *cmd, char *tbuf)
 	char *space = strchr (cmd, ' ');
 	if (space && space != cmd)
 	{
-		if (((space[-1] == ':') || (space[-1] == prefs.nick_suffix[0])) && (space - 1 != cmd))
+		if (space[-1] == prefs.nick_suffix[0] && space - 1 != cmd)
 		{
 			len = space - cmd - 1;
 			if (len < NICKLEN)
@@ -3377,6 +3382,9 @@ xit:
 static int
 handle_user_input (session *sess, char *text, int history, int nocommand)
 {
+	if (*text == '\0')
+		return 1;
+
 	if (history)
 		history_add (&sess->history, text);
 
