@@ -196,9 +196,10 @@ plugin_list_add (xchat_context *ctx, char *filename, const char *name,
 	return pl;
 }
 
-static void
+static void *
 xchat_dummy (xchat_plugin *ph)
 {
+	return NULL;
 }
 
 #ifdef WIN32
@@ -227,6 +228,7 @@ plugin_add (session *sess, char *filename, void *handle, void *init_func,
 
 	if (!fake)
 	{
+		/* win32 uses these because it doesn't have --export-dynamic! */
 		pl->xchat_hook_command = xchat_hook_command;
 		pl->xchat_hook_server = xchat_hook_server;
 		pl->xchat_hook_print = xchat_hook_print;
@@ -252,13 +254,16 @@ plugin_add (session *sess, char *filename, void *handle, void *init_func,
 		pl->xchat_plugingui_add = xchat_plugingui_add;
 		pl->xchat_plugingui_remove = xchat_plugingui_remove;
 		pl->xchat_emit_print = xchat_emit_print;
-		/* incase new plugins are loaded on older xchat */
 #ifdef WIN32
 		pl->xchat_read_fd = (void *) xchat_read_fd;
 #else
 		pl->xchat_read_fd = xchat_dummy;
 #endif
 		pl->xchat_list_time = xchat_list_time;
+		pl->xchat_gettext = xchat_gettext;
+		pl->xchat_send_modes = xchat_send_modes;
+
+		/* incase new plugins are loaded on older xchat */
 		pl->xchat_dummy6 = xchat_dummy;
 		pl->xchat_dummy5 = xchat_dummy;
 		pl->xchat_dummy4 = xchat_dummy;
@@ -385,6 +390,7 @@ plugin_load (session *sess, char *filename, char *arg)
 
 	/* find the plugin's deinit routine, if any */
 	deinit_func = dlsym (handle, "xchat_plugin_deinit");
+	error = dlerror ();
 #endif
 
 	/* add it to our linked list */
@@ -403,9 +409,7 @@ plugin_auto_load_cb (char *filename)
 	pMsg = plugin_load (ps, filename, NULL);
 	if (pMsg)
 	{
-		PrintText (ps, "AutoLoad failed for: ");
-		PrintText (ps, filename);
-		PrintText (ps, "\n");
+		PrintTextf (ps, "AutoLoad failed for: %s\n", filename);
 		PrintText (ps, pMsg);
 	}
 }
@@ -417,10 +421,15 @@ plugin_auto_load (session *sess)
 #ifdef WIN32
 	for_files ("./plugins", "*.dll", plugin_auto_load_cb);
 	for_files (get_xdir_fs (), "*.dll", plugin_auto_load_cb);
+#else
+#if defined(__hpux)
+	for_files (XCHATLIBDIR"/plugins", "*.sl", plugin_auto_load_cb);
+	for_files (get_xdir_fs (), "*.sl", plugin_auto_load_cb);
 #else
 	for_files (XCHATLIBDIR"/plugins", "*.so", plugin_auto_load_cb);
 	for_files (get_xdir_fs (), "*.so", plugin_auto_load_cb);
 #endif
+#endif
 }
 
 #endif
@@ -906,8 +915,20 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 	case 0x14f51cd8: /* version */
 		return VERSION;
 
+	case 0x6d3431b5: /* win_status */
+		switch (fe_gui_info (sess, 0))	/* check window status */
+		{
+		case 0: return "normal";
+		case 1: return "active";
+		case 2: return "hidden";
+		}
+		return NULL;
+
 	case 0xdd9b1abd:	/* xchatdir */
 		return get_xdir_utf8 ();
+
+	case 0xe33f6c4a:	/* xchatdirfs */
+		return get_xdir_fs ();
 	}
 
 	return NULL;
@@ -1035,8 +1056,9 @@ xchat_list_fields (xchat_plugin *ph, const char *name)
 	};
 	static const char *channels_fields[] =
 	{
-		"schannel",	"pcontext", "iflags", "iid", "snetwork", "sserver",
-		"itype", "iusers", NULL
+		"schannel",	"schantypes", "pcontext", "iflags", "iid", "imaxmodes",
+		"snetwork", "snickmodes", "snickprefixes", "sserver", "itype", "iusers",
+		NULL
 	};
 	static const char *ignore_fields[] =
 	{
@@ -1112,12 +1134,18 @@ xchat_list_str (xchat_plugin *ph, xchat_list *xlist, const char *name)
 		{
 		case 0x2c0b7d03: /* channel */
 			return ((session *)data)->channel;
+		case 0x577e0867: /* chantypes */
+			return ((session *)data)->server->chantypes;
 		case 0x38b735af: /* context */
 			return data;	/* this is a session * */
+		case 0x6de15a2e: /* network */
+			return ((session *)data)->server->networkname;
+		case 0x8455e723: /* nickprefixes */
+			return ((session *)data)->server->nick_prefixes;
+		case 0x829689ad: /* nickmodes */
+			return ((session *)data)->server->nick_modes;
 		case 0xca022f43: /* server */
 			return ((session *)data)->server->servername;
-		case 0x6de15a2e:        /* network */
-			return ((session *)data)->server->networkname;
 		}
 		break;
 
@@ -1170,6 +1198,7 @@ xchat_list_int (xchat_plugin *ph, xchat_list *xlist, const char *name)
 {
 	guint32 hash = str_hash (name);
 	gpointer data = xlist->pos->data;
+	int tmp;
 
 	switch (xlist->type)
 	{
@@ -1209,11 +1238,18 @@ xchat_list_int (xchat_plugin *ph, xchat_list *xlist, const char *name)
 		case 0xd1b:	/* id */
 			return ((struct session *)data)->server->id;
 		case 0x5cfee87:	/* flags */
-			return ((((struct session *)data)->server->connected) & 1) ||
-					 ((((struct session *)data)->server->connecting << 1) & 1) ||
-					 ((((struct session *)data)->server->is_away << 2) & 2) ||
-					 ((((struct session *)data)->server->end_of_motd << 3) & 4) ||
-					 ((((struct session *)data)->server->have_whox << 4) & 8);
+			tmp = ((struct session *)data)->server->have_whox;   /* bit 4 */
+			tmp <<= 1;
+			tmp |= ((struct session *)data)->server->end_of_motd;/* 3 */
+			tmp <<= 1;
+			tmp |= ((struct session *)data)->server->is_away;    /* 2 */
+			tmp <<= 1;
+			tmp |= ((struct session *)data)->server->connecting; /* 1 */ 
+			tmp <<= 1;
+			tmp |= ((struct session *)data)->server->connected;  /* 0 */
+			return tmp;
+		case 0x1916144c: /* maxmodes */
+			return ((struct session *)data)->server->modes_per_line;
 		case 0x368f3a:	/* type */
 			return ((struct session *)data)->type;
 		case 0x6a68e08: /* users */
@@ -1290,3 +1326,19 @@ xchat_emit_print (xchat_plugin *ph, const char *event_name, ...)
 
 	return i;
 }
+
+char *
+xchat_gettext (xchat_plugin *ph, const char *msgid)
+{
+	/* so that plugins can use xchat's internal gettext strings. */
+	/* e.g. The EXEC plugin uses this on Windows. */
+	return _(msgid);
+}
+
+void
+xchat_send_modes (xchat_plugin *ph, const char **targets, int ntargets, int modes_per_line, char sign, char mode)
+{
+	char tbuf[514];	/* modes.c needs 512 + null */
+
+	send_channel_modes (ph->context, tbuf, targets, 0, ntargets, sign, mode, modes_per_line);
+}
