@@ -80,16 +80,24 @@ struct _xchat_plugin
 			void *handle);
 	int (*xchat_emit_print) (xchat_plugin *ph,
 			const char *event_name, ...);
-	void (*xchat_read_fd) (xchat_plugin *ph);
+	void *(*xchat_read_fd) (xchat_plugin *ph);
 	time_t (*xchat_list_time) (xchat_plugin *ph,
 		xchat_list *xlist,
 		const char *name);
-	void (*xchat_dummy6) (xchat_plugin *ph);
-	void (*xchat_dummy5) (xchat_plugin *ph);
-	void (*xchat_dummy4) (xchat_plugin *ph);
-	void (*xchat_dummy3) (xchat_plugin *ph);
-	void (*xchat_dummy2) (xchat_plugin *ph);
-	void (*xchat_dummy1) (xchat_plugin *ph);
+	char *(*xchat_gettext) (xchat_plugin *ph,
+		const char *msgid);
+	void (*xchat_send_modes) (xchat_plugin *ph,
+		  const char **targets,
+		  int ntargets,
+		  int modes_per_line,
+		  char sign,
+		  char mode);
+	void *(*xchat_dummy6) (xchat_plugin *ph);
+	void *(*xchat_dummy5) (xchat_plugin *ph);
+	void *(*xchat_dummy4) (xchat_plugin *ph);
+	void *(*xchat_dummy3) (xchat_plugin *ph);
+	void *(*xchat_dummy2) (xchat_plugin *ph);
+	void *(*xchat_dummy1) (xchat_plugin *ph);
 	/* PRIVATE FIELDS! */
 	void *handle;		/* from dlopen */
 	char *filename;	/* loaded from */
