@@ -286,9 +286,9 @@ irc_raw (server *serv, char *raw)
 static void
 channel_date (session *sess, char *chan, char *timestr)
 {
-	long n = atol (timestr);
-	char *tim = ctime (&n);
-	tim[19] = 0;	/* get rid of the \n */
+	time_t timestamp = (time_t) atol (timestr);
+	char *tim = ctime (&timestamp);
+	tim[24] = 0;	/* get rid of the \n */
 	EMIT_SIGNAL (XP_TE_CHANDATE, sess, chan, tim, NULL, NULL, 0);
 }
 
@@ -303,6 +303,17 @@ process_numeric (session * sess, int n,
 	{
 	case 1:
 		inbound_login_start (sess, word[3], word[1]);
+		/* if network is PTnet then you must get your IP address
+			from "001" server message */
+		if ((strncmp(word[7], "PTnet", 5) == 0) &&
+			(strncmp(word[8], "IRC", 3) == 0) &&
+			(strncmp(word[9], "Network", 7) == 0) &&
+			(strchr(word[10], '@') != NULL))
+		{
+			serv->use_who = FALSE;
+			if (prefs.ip_from_server)
+				inbound_foundip (sess, strchr(word[10], '@')+1);
+		}
 		goto def;
 
 	case 4:	/* check the ircd type */
@@ -364,7 +375,7 @@ process_numeric (session * sess, int n,
 
 	case 317:
 		{
-			long nn = atol (word[6]);
+			time_t timestamp = (time_t) atol (word[6]);
 			long idle = atol (word[5]);
 			char *tim;
 			char outbuf[64];
@@ -372,12 +383,12 @@ process_numeric (session * sess, int n,
 			snprintf (outbuf, sizeof (outbuf),
 						"%02ld:%02ld:%02ld", idle / 3600, (idle / 60) % 60,
 						idle % 60);
-			if (nn == 0)
+			if (timestamp == 0)
 				EMIT_SIGNAL (XP_TE_WHOIS4, serv->server_session, word[4],
 								 outbuf, NULL, NULL, 0);
 			else
 			{
-				tim = ctime (&nn);
+				tim = ctime (&timestamp);
 				tim[19] = 0; 	/* get rid of the \n */
 				EMIT_SIGNAL (XP_TE_WHOIS4T, serv->server_session, word[4],
 								 outbuf, tim, NULL, 0);
