@@ -289,8 +289,8 @@ server_inline (server *serv, char *line, int len)
 		{
 			char *conv_line; /* holds a copy of the original string */
 			int conv_len; /* tells g_convert how much of line to convert */
-			int utf_len;
-			int read_len;
+			gsize utf_len;
+			gsize read_len;
 			GError *err;
 			gboolean retry;
 
@@ -439,7 +439,7 @@ server_connected (server * serv)
 
 #ifdef WIN32
 	if (prefs.identd)
-		identd_start ();
+		identd_start ((serv->username) ? serv->username : prefs.username);
 #else
 	sprintf (outbuf, "%s/auth/xchat_auth", g_get_home_dir ());
 	if (access (outbuf, X_OK) == 0)
@@ -470,6 +470,19 @@ server_connected (server * serv)
 	set_server_name (serv, serv->servername);
 }
 
+#ifdef WIN32
+
+static gboolean
+server_close_pipe (int *pipefd)	/* see comments below */
+{
+	close (pipefd[0]);	/* close WRITE end first to cause an EOF on READ */
+	close (pipefd[1]);	/* in giowin32, and end that thread. */
+	free (pipefd);
+	return FALSE;
+}
+
+#endif
+
 static void
 server_stopconnecting (server * serv)
 {
@@ -483,12 +496,20 @@ server_stopconnecting (server * serv)
 	/* kill the child process trying to connect */
 	kill (serv->childpid, SIGKILL);
 	waitpid (serv->childpid, NULL, 0);
-#else
-	PostThreadMessage (serv->childpid, WM_QUIT, 0, 0);
-#endif
 
 	close (serv->childwrite);
 	close (serv->childread);
+#else
+	PostThreadMessage (serv->childpid, WM_QUIT, 0, 0);
+
+	{
+		/* if we close the pipe now, giowin32 will crash. */
+		int *pipefd = malloc (sizeof (int) * 2);
+		pipefd[0] = serv->childwrite;
+		pipefd[1] = serv->childread;
+		g_idle_add ((GSourceFunc)server_close_pipe, pipefd);
+	}
+#endif
 
 #ifdef USE_OPENSSL
 	if (serv->ssl_do_connect_tag)
@@ -504,7 +525,7 @@ server_stopconnecting (server * serv)
 }
 
 #ifdef USE_OPENSSL
-#define	SSLTMOUT	10				  /* seconds */
+#define	SSLTMOUT	90				  /* seconds */
 static void
 ssl_cb_info (SSL * s, int where, int ret)
 {
@@ -560,6 +581,9 @@ ssl_do_connect (server * serv)
 			EMIT_SIGNAL (XP_TE_CONNFAIL, serv->server_session, buf, NULL,
 							 NULL, NULL, 0);
 
+			if (ERR_GET_REASON (err) == SSL_R_WRONG_VERSION_NUMBER)
+				PrintText (serv->server_session, _("Are you sure this is a SSL capable server and port?\n"));
+
 			server_cleanup (serv);
 
 			if (prefs.autoreconnectonfail)
@@ -628,7 +652,7 @@ ssl_do_connect (server * serv)
 		}
 
 		chiper_info = _SSL_get_cipher_info (serv->ssl);	/* static buffer */
-		snprintf (buf, sizeof (buf), "* Chiper info:");
+		snprintf (buf, sizeof (buf), "* Cipher info:");
 		EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, buf, NULL, NULL, NULL,
 						 0);
 		snprintf (buf, sizeof (buf), "  Version: %s, cipher %s (%u bits)",
@@ -648,6 +672,7 @@ ssl_do_connect (server * serv)
 		case X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE:
 		case X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT:
 		case X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN:
+		case X509_V_ERR_CERT_HAS_EXPIRED:
 			if (serv->accept_invalid_cert)
 			{
 				snprintf (buf, sizeof (buf), "* Verify E: %s.? (%d) -- Ignored",
@@ -936,6 +961,7 @@ server_cleanup (server * serv)
 	{
 		close_socket (serv->sok);
 		serv->connected = FALSE;
+		serv->end_of_motd = FALSE;
 		return 2;
 	}
 
@@ -1001,6 +1027,11 @@ server_disconnect (session * sess, int sendquit, int err)
 	serv->motd_skipped = FALSE;
 	serv->no_login = FALSE;
 	serv->servername[0] = 0;
+	if (serv->networkname)
+	{
+		free (serv->networkname);
+		serv->networkname = NULL;
+	}
 	serv->lag_sent = 0;
 
 	notify_cleanup ();
@@ -1053,7 +1084,7 @@ traverse_socks5 (int sok, char *serverAddr, int port)
 	struct sock5_connect1 sc1;
 	unsigned char *sc2;
 	unsigned int packetlen, addrlen;
-	unsigned char buf[10];
+	unsigned char buf[260];
 
 	sc1.version = 5;
 	sc1.nmethods = 1;
@@ -1094,7 +1125,7 @@ traverse_socks5 (int sok, char *serverAddr, int port)
 	{
 		if (recv (sok, buf, 1, 0) != 1)
 			return 1;
-		packetlen = buf[0] + 2;
+		packetlen = buf[0] + 2;	/* can't exceed 260 */
 		if (recv (sok, buf, packetlen, 0) != packetlen)
 			return 1;
 	}
@@ -1113,14 +1144,72 @@ traverse_wingate (int sok, char *serverAddr, int port)
 	return 0;
 }
 
+/* stuff for HTTP auth is here */
+
+static void
+three_to_four (char *from, char *to)
+{
+	static const char tab64[64]=
+	{
+		'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P',
+		'Q','R','S','T','U','V','W','X','Y','Z','a','b','c','d','e','f',
+		'g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v',
+		'w','x','y','z','0','1','2','3','4','5','6','7','8','9','+','/'
+	};
+
+	to[0] = tab64 [ (from[0] >> 2) & 63 ];
+	to[1] = tab64 [ ((from[0] << 4) | (from[1] >> 4)) & 63 ];
+	to[2] = tab64 [ ((from[1] << 2) | (from[2] >> 6)) & 63 ];
+	to[3] = tab64 [ from[2] & 63 ];
+};
+
+static void
+base64_encode (char *to, char *from, unsigned int len)
+{
+	while (len >= 3)
+	{
+		three_to_four (from, to);
+		len -= 3;
+		from += 3;
+		to += 4;
+	}
+	if (len)
+	{
+		char three[3]={0,0,0};
+		int i=0;
+		for (i=0;i<len;i++)
+			three[i] = *from++;
+		three_to_four (three, to);
+		if (len == 1)
+			to[2] = to[3] = '=';
+		else if (len == 2)
+			to[3] = '=';
+		to += 4;
+	};
+	to[0] = 0;
+}
+
 static int
 traverse_http (int sok, char *serverAddr, int port)
 {
-	char buf[128];
+	char buf[256];
+	char auth_data[128];
+	char auth_data2[68];
+	int n, n2;
 
-	snprintf (buf, sizeof (buf), "CONNECT %s:%d HTTP/1.0\r\n\r\n",
+	n = snprintf (buf, sizeof (buf), "CONNECT %s:%d HTTP/1.0\r\n",
 					  serverAddr, port);
-	send (sok, buf, strlen (buf), 0);
+	if (prefs.proxy_auth)
+	{
+		n2 = snprintf (auth_data2, sizeof (auth_data2), "%s:%s",
+							prefs.proxy_user, prefs.proxy_pass);
+		base64_encode (auth_data, auth_data2, n2);
+		n += snprintf (buf+n, sizeof (buf)-n, "Proxy-Authorization: Basic %s\r\n", auth_data);
+	}
+	n += snprintf (buf+n, sizeof (buf)-n, "\r\n");
+	send (sok, buf, n, 0);
+
+	/* dont eat this, it's actually informative to display */
 #if 0
 	waitline (sok, buf, sizeof (buf)); /* FIXME: win32 cant read() sok */
 	/* "HTTP/1.0 200 OK" */
@@ -1174,9 +1263,7 @@ server_child (server * serv)
 	char *proxy_ip = NULL;
 	char *local_ip;
 	int connect_port;
-	FILE *fd;
-
-	fd = fdopen (serv->childwrite, "w");
+	char buf[512];
 
 	ns_server = net_store_new ();
 
@@ -1187,26 +1274,26 @@ server_child (server * serv)
 		local_ip = net_resolve (ns_local, prefs.hostname, 0, &real_hostname);
 		if (local_ip != NULL)
 		{
-			fprintf (fd, "5\n%s\n", local_ip);
+			snprintf (buf, sizeof (buf), "5\n%s\n", local_ip);
+			write (serv->childwrite, buf, strlen (buf));
 			net_bind (ns_local, serv->sok4, serv->sok6);
 		} else
 		{
-			fprintf (fd, "7\n");
+			write (serv->childwrite, "7\n", 2);
 		}
 		net_store_destroy (ns_local);
-		fflush (fd);
 	}
 
 	/* first resolve where we want to connect to */
 	if (!serv->dont_use_proxy && prefs.proxy_host[0] && prefs.proxy_type > 0)
 	{
-		fprintf (fd, "9\n%s\n", prefs.proxy_host);
-		fflush (fd);
+		snprintf (buf, sizeof (buf), "9\n%s\n", prefs.proxy_host);
+		write (serv->childwrite, buf, strlen (buf));
 		ip = net_resolve (ns_server, prefs.proxy_host, prefs.proxy_port,
 								&real_hostname);
 		if (!ip)
 		{
-			fprintf (fd, "1\n");
+			write (serv->childwrite, "1\n", 2);
 			goto xit;
 		}
 		connect_port = prefs.proxy_port;
@@ -1218,7 +1305,7 @@ server_child (server * serv)
 			proxy_ip = net_resolve (ns_proxy, hostname, port, &real_hostname);
 			if (!proxy_ip)
 			{
-				fprintf (fd, "1\n");
+				write (serv->childwrite, "1\n", 2);
 				goto xit;
 			}
 		} else						  /* otherwise we can just use the hostname */
@@ -1228,20 +1315,22 @@ server_child (server * serv)
 		ip = net_resolve (ns_server, hostname, port, &real_hostname);
 		if (!ip)
 		{
-			fprintf (fd, "1\n");
+			write (serv->childwrite, "1\n", 2);
 			goto xit;
 		}
 		connect_port = port;
 	}
 
-	fprintf (fd, "3\n%s\n%s\n%d\n", real_hostname, ip, connect_port);
-	fflush (fd);
+	snprintf (buf, sizeof (buf), "3\n%s\n%s\n%d\n",
+				 real_hostname, ip, connect_port);
+	write (serv->childwrite, buf, strlen (buf));
 
 	error = net_connect (ns_server, serv->sok4, serv->sok6, &sok);
 
 	if (error != 0)
 	{
-		fprintf (fd, "2\n%d\n", sock_error ());
+		snprintf (buf, sizeof (buf), "2\n%d\n", sock_error ());
+		write (serv->childwrite, buf, strlen (buf));
 	} else
 	{
 		/* connect succeeded */
@@ -1249,24 +1338,23 @@ server_child (server * serv)
 		{
 			switch (traverse_proxy (sok, proxy_ip, port))
 			{
-			case 0:
-				fprintf (fd, "4\n%d\n", sok);	/* success */
+			case 0:	/* success */
+				snprintf (buf, sizeof (buf), "4\n%d\n", sok);	/* success */
+				write (serv->childwrite, buf, strlen (buf));
 				break;
-			case 1:
-				fprintf (fd, "8\n");	  /* socks traversal failed */
+			case 1:	/* socks traversal failed */
+				write (serv->childwrite, "8\n", 2);
 				break;
 			}
 		} else
 		{
-			fprintf (fd, "4\n%d\n", sok);
+			snprintf (buf, sizeof (buf), "4\n%d\n", sok);	/* success */
+			write (serv->childwrite, buf, strlen (buf));
 		}
 	}
 
 xit:
 
-	/* 'fd' is leaked on win32 */
-	fflush (fd);
-
 #if defined (USE_IPV6) || defined (WIN32)
 	/* this is probably not needed */
 	net_store_destroy (ns_server);
@@ -1283,7 +1371,6 @@ xit:
 		free (ip);
 	if (real_hostname)
 		free (real_hostname);
-	/*fclose (fd);*/
 #endif
 
 	return 0;
