@@ -57,22 +57,28 @@ static const struct defaultserver def[] =
 
 	{"AfterNET",	0},
 	{0,			"irc.afternet.org"},
-	{0,			"ic5.eu.afternet.org"},
-	{0,			"baltimore.md.us.afternet.org"},
-	{0,			"boston.afternet.org"},
+	{0,			"us.afternet.org"},
+	{0,			"eu.afternet.org"},
 
 	{"Aitvaras",	0},
 #ifdef USE_IPV6
+#ifdef USE_OPENSSL
+	{0,			"irc6.ktu.lt/+7668"},
+#endif
 	{0,			"irc6.ktu.lt/7666"},
 #endif
-	{0,			"irc.delfi.lt"},
+#ifdef USE_OPENSSL
+	{0,			"irc.data.lt/+6668"},
+#endif
+	{0,			"irc.data.lt"},
 	{0,			"irc.ktu.lt"},
 	{0,			"irc.le.lt"},
-	{0,			"irc.takas.lt"},
 	{0,			"irc.omnitel.net"},
+	{0,			"irc.takas.lt"},
 	{0,			"irc.5ci.net"},
 	{0,			"irc.interneka.lt"},
 	{0,			"irc.elekta.lt"},
+	{0,			"irc.if.lt"},
 
 	{"AmigaNet",	0},
 	{0,			"linux.us.amiganet.org"},
@@ -132,9 +138,6 @@ static const struct defaultserver def[] =
 	{0,			"unix.coolchat.net"},
 	{0,			"toronto.coolchat.net"},
 
-	{"CyberArmy",	0},
-	{0,			"irc.cyberarmy.com"},
-
 	{"DALnet",	0},
 	{0,			"irc.dal.net"},
 	{0,			"irc.eu.dal.net"},
@@ -172,6 +175,9 @@ static const struct defaultserver def[] =
 	{0,			"irc.inn.at.euirc.net"},
 	{0,			"irc.bas.ch.euirc.net"},
 
+	{"EU-IRC",	0},
+	{0,			"irc.eu-irc.net"},
+
 	{"FDFNet",	0},
 	{0,			"irc.fdfnet.net"},
 	{0,			"irc.eu.fdfnet.net"},
@@ -258,11 +264,6 @@ static const struct defaultserver def[] =
 	{0,			"irc.canadian.net"},
 	{0,			"irc.irctoo.net"},
 
-	{"KewlNet",	0},
-	{0,			"random.irc.kewl.org"},
-	{0,			"la.defense.fr.eu.kewl.org"},
-	{0,			"nanterre.fr.eu.kewl.org"},
-
 	{"Librenet",	0},
 	{0,			"irc.librenet.net"},
 	{0,			"ielf.fr.librenet.net"},
@@ -278,6 +279,9 @@ static const struct defaultserver def[] =
 	{"MagicStar",	0},
 	{0,			"irc.magicstar.net"},
 
+	{"Majistic",	0},
+	{0,			"irc.majistic.net"},
+
 	{"NeverNET",	0},
 	{0,			"irc.nevernet.net"},
 	{0,			"imagine.nevernet.net"},
@@ -377,6 +381,16 @@ static const struct defaultserver def[] =
 	{"RebelChat",	0},
 	{0,			"irc.rebelchat.org"},
 
+	{"Recycled-IRC",  0},
+	{0,			"irc.recycled-irc.org"},
+	{0,			"vermin.recycled-irc.org"},
+	{0,			"waste.recycled-irc.org"},
+	{0,			"lumber.recycled-irc.org"},
+	{0,			"trash.recycled-irc.org"},
+	{0,			"unwashed.recycled-irc.org"},
+	{0,			"garbage.recycled-irc.org"},
+	{0,			"dustbin.recycled-irc.org"},
+
 	{"RedeBrasilIRC",	0},
 	{0,			"irc.redebrasilirc.org"},
 
@@ -408,13 +422,14 @@ static const struct defaultserver def[] =
 	{0,			"eu.spidernet.org"},
 	{0,			"irc.spidernet.org"},
 
-	{"StarChat",	0},
+	{"StarChat", 0},
 	{0,			"irc.starchat.net"},
-	{0,			"galatea.starchat.net"},
-	{0,			"stargate.starchat.net"},
-	{0,			"powerzone.starchat.net"},
-	{0,			"utopia.starchat.net"},
-	{0,			"cairns.starchat.net"},
+	{0,			"gainesville.starchat.net"},
+	{0,			"freebsd.starchat.net"},
+	{0,			"sunset.starchat.net"},
+	{0,			"revenge.starchat.net"},
+	{0,			"tahoma.starchat.net"},
+	{0,			"neo.starchat.net"},
 
 	{"SubCultNet",	0},
 	{0,			"irc.subcult.ch"},
@@ -443,6 +458,10 @@ static const struct defaultserver def[] =
 	{"Unsecurity",	0},
 	{0,			"irc.unsecurity.org"},
 
+	{"Worldnet",		0},
+	{0,			"irc.worldnet.net"},
+	{0,			"irc.fr.worldnet.net"},
+
 	{"Xentonix.net",	0},
 	{0,			"irc.ffm.de.eu.xentonix.net"},
 	{0,			"irc.brs.de.eu.xentonix.net"},
@@ -487,7 +506,7 @@ servlist_connect (session *sess, ircnet *net)
 	/* incase a protocol switch is added to the servlist gui */
 	server_fill_her_up (sess->server);
 
-	if (net->autojoin)
+	if (net->autojoin && !sess->server->recondelay_tag)
 		safe_strcpy (sess->willjoinchannel, net->autojoin,
 						 sizeof (sess->willjoinchannel));
 	if (net->pass)
@@ -537,7 +556,19 @@ servlist_connect (session *sess, ircnet *net)
 	if (port)
 	{
 		*port = 0;
-		serv->connect (serv, ircserv->hostname, atoi (port + 1), FALSE);
+
+		/* support "+port" to indicate SSL (like mIRC does) */
+		if (port[1] == '+')
+		{
+#ifdef USE_OPENSSL
+			serv->use_ssl = TRUE;
+#endif
+			serv->connect (serv, ircserv->hostname, atoi (port + 2), FALSE);
+		} else
+		{
+			serv->connect (serv, ircserv->hostname, atoi (port + 1), FALSE);
+		}
+
 		*port = '/';
 	} else
 		serv->connect (serv, ircserv->hostname, 6667, FALSE);
@@ -816,6 +847,11 @@ servlist_load_defaults (void)
 		if (def[i].network)
 		{
 			net = servlist_net_add (def[i].network, def[i].host);
+#ifdef WIN32
+			/* Windows gets UTF-8 for new users. Unix gets "System Default",
+				which is often UTF-8 anyway! */
+			net->encoding = strdup ("UTF-8");
+#endif
 			if (def[i].channel)
 				net->autojoin = strdup (def[i].channel);
 			if (!strcmp (def[i].network, "ChatJunkies"))
@@ -886,7 +922,7 @@ servlist_load (void)
 			}
 		}
 		if (buf[0] == 'N')
-			net = servlist_net_add (buf + 2, /* comment */ "");
+			net = servlist_net_add (buf + 2, /* comment */ NULL);
 	}
 	fclose (fp);
 
