@@ -27,6 +27,7 @@
 #include <sys/stat.h>
 
 #include "xchat.h"
+#include <glib/ghash.h>
 #include "cfgfiles.h"
 #include "plugin.h"
 #include "fe.h"
@@ -34,7 +35,9 @@
 #include "outbound.h"
 #include "xchatc.h"
 #include "text.h"
-
+#ifdef WIN32
+#include <windows.h>
+#endif
 
 struct pevt_stage1
 {
@@ -154,7 +157,7 @@ q3link (char *word)
 
 	if ((s = strchr (word,':')) != NULL)
 	{
-		for (i = 0; i < (int) ((unsigned long)s - (unsigned long)word); i++)
+		for (i = 0; i < s - word; i++)
 		{
 			if (word[i] == '.')
 				d++;
@@ -196,7 +199,7 @@ text_word_check (char *word)
 	if(q3link(word))
 		return WORD_URL;
 
-	if ((word[0] == '@' || word[0] == '+') && word[1] == '#')
+	if ((word[0] == '@' || word[0] == '+' || word[0] == '^' || word[0] == '%' || word[0] == '*' ) && word[1] == '#')
 		return WORD_CHANNEL;
 
 	if ((word[0] == '#' || word[0] == '&') && word[1] != '#' && word[1] != 0)
@@ -236,7 +239,7 @@ text_word_check (char *word)
 	dot = strrchr (word, '.');
 	if (at && dot)
 	{
-		if ((unsigned long) at < (unsigned long) dot)
+		if (at < dot)
 		{
 			if (strchr (word, '*'))
 				return WORD_HOST;
@@ -445,9 +448,21 @@ log_open (session *sess)
 int
 get_stamp_str (char *fmt, time_t tim, char **ret)
 {
+	char *loc = NULL;
 	char dest[128];
 	gsize len;
 
+	/* strftime wants the format string in LOCALE! */
+	if (!prefs.utf8_locale)
+	{
+		const gchar *charset;
+
+		g_get_charset (&charset);
+		loc = g_convert_with_fallback (fmt, -1, charset, "UTF-8", "?", 0, 0, 0);
+		if (loc)
+			fmt = loc;
+	}
+
 	len = strftime (dest, sizeof (dest), fmt, localtime (&tim));
 	if (len)
 	{
@@ -457,6 +472,9 @@ get_stamp_str (char *fmt, time_t tim, char **ret)
 			*ret = g_locale_to_utf8 (dest, len, 0, &len, 0);
 	}
 
+	if (loc)
+		g_free (loc);
+
 	return len;
 }
 
@@ -938,7 +956,7 @@ static char *pevt_nickclash_help[] = {
 };
 
 static char *pevt_connfail_help[] = {
-	N_("Error String"),
+	N_("Error"),
 };
 
 static char *pevt_connect_help[] = {
@@ -974,7 +992,7 @@ static char *pevt_dccchaterr_help[] = {
 	N_("Nickname"),
 	N_("IP address"),
 	N_("Port"),
-	N_("Error name"),
+	N_("Error"),
 };
 
 static char *pevt_dccstall_help[] = {
@@ -985,13 +1003,14 @@ static char *pevt_dccstall_help[] = {
 
 static char *pevt_generic_file_help[] = {
 	N_("Filename"),
+	N_("Error"),
 };
 
 static char *pevt_dccrecverr_help[] = {
 	N_("Filename"),
 	N_("Destination filename"),
 	N_("Nickname"),
-	N_("Error name"),
+	N_("Error"),
 };
 
 static char *pevt_dccrecvcomp_help[] = {
@@ -1004,7 +1023,7 @@ static char *pevt_dccrecvcomp_help[] = {
 static char *pevt_dccconfail_help[] = {
 	N_("DCC Type"),
 	N_("Nickname"),
-	N_("Error string"),
+	N_("Error"),
 };
 
 static char *pevt_dccchatcon_help[] = {
@@ -1021,7 +1040,7 @@ static char *pevt_dcccon_help[] = {
 static char *pevt_dccsendfail_help[] = {
 	N_("Filename"),
 	N_("Nickname"),
-	N_("Error name"),
+	N_("Error"),
 };
 
 static char *pevt_dccsendcomp_help[] = {
@@ -1289,7 +1308,33 @@ pevent_load (char *filename)
 		{
 			if (text)
 				free (text);
-			text = strdup (ofs);
+
+			/* This allows updating of old strings. We don't use new defaults
+				if the user has customized the strings (.e.g a text theme).
+				Hash of the old default is enough to identify and replace it.
+				This only works in English. */
+
+			switch (g_str_hash (ofs))
+			{
+			case 0x526743a4:
+		/* %C08,02 Hostmask                  PRIV NOTI CHAN CTCP INVI UNIG %O */
+				text = strdup (te[XP_TE_IGNOREHEADER].def);
+				break;
+
+			case 0xe91bc9c2:
+		/* %C08,02                                                         %O */
+				text = strdup (te[XP_TE_IGNOREFOOTER].def);
+				break;
+
+			case 0x1fbfdf22:
+		/* -%C10-%C11-%O$tDCC RECV: Cannot open $1 for writing - aborting. */
+				text = strdup (te[XP_TE_DCCFILEERR].def);
+				break;
+
+			default:
+				text = strdup (ofs);
+			}
+
 			continue;
 		} else if (strcmp (buf, "event_sound") == 0)
 		{
@@ -1595,9 +1640,23 @@ play_wave (const char *file)
 {
 	char buf[512];
 	char wavfile[512];
+	char *file_fs;
+
+	/* the pevents GUI editor triggers this after removing a soundfile */
+	if (!file[0])
+		return;
 
 	memset (buf, 0, sizeof (buf));
 	memset (wavfile, 0, sizeof (wavfile));
+
+#ifdef WIN32
+	/* check for fullpath, windows style */
+	if (strlen (file) > 3 &&
+		 file[1] == ':' && (file[2] == '\\' || file[2] == '/') )
+	{
+		strncpy (wavfile, file, sizeof (wavfile));
+	} else
+#endif
 	if (file[0] != '/')
 	{
 		snprintf (wavfile, sizeof (wavfile), "%s/%s", prefs.sounddir, file);
@@ -1605,16 +1664,31 @@ play_wave (const char *file)
 	{
 		strncpy (wavfile, file, sizeof (wavfile));
 	}
-	if (access (wavfile, R_OK) == 0)
+
+	file_fs = g_filename_from_utf8 (wavfile, -1, 0, 0, 0);
+	if (!file_fs)
+		return;
+
+	if (access (file_fs, R_OK) == 0)
 	{
-		snprintf (buf, sizeof (buf), "%s %s", prefs.soundcmd, wavfile);
-		buf[sizeof (buf) - 1] = '\0';
-		xchat_exec (buf);
+#ifdef WIN32
+		if (strcmp (prefs.soundcmd, "esdplay") == 0)
+		{
+			PlaySound (file_fs, NULL, SND_NODEFAULT|SND_FILENAME|SND_ASYNC);
+		} else
+#endif
+		{
+			snprintf (buf, sizeof (buf), "%s %s", prefs.soundcmd, file_fs);
+			buf[sizeof (buf) - 1] = '\0';
+			xchat_exec (buf);
+		}
 	} else
 	{
-		snprintf (buf, sizeof (buf), "Cannot read %s", wavfile);
+		snprintf (buf, sizeof (buf), _("Cannot read sound file:\n%s"), wavfile);
 		fe_message (buf, FALSE);
 	}
+
+	g_free (file_fs);
 }
 
 /* called by EMIT_SIGNAL macro */
