@@ -34,39 +34,39 @@ enum
 	XP_TE_DCCSENDFAIL,		XP_TE_DCCSENDOFFER,
 	XP_TE_DCCSTALL,		XP_TE_DCCTOUT,
 	XP_TE_DELNOTIFY,		XP_TE_DISCON,
-	XP_TE_FOUNDIP,		XP_TE_IGNOREADD,
-	XP_TE_IGNORECHANGE,		XP_TE_IGNOREFOOTER,
-	XP_TE_IGNOREHEADER,		XP_TE_IGNOREREMOVE,
-	XP_TE_IGNOREEMPTY,		XP_TE_INVITE,
-	XP_TE_INVITED,		XP_TE_JOIN,
-	XP_TE_KEYWORD,		XP_TE_KICK,
-	XP_TE_KILL,		XP_TE_MSGSEND,
-	XP_TE_MOTD,		XP_TE_MOTDSKIP,
-	XP_TE_NICKCLASH,		XP_TE_NICKFAIL,
-	XP_TE_NODCC,		XP_TE_NOCHILD,
-	XP_TE_NOTICE,		XP_TE_NOTICESEND,
-	XP_TE_NOTIFYEMPTY,		XP_TE_NOTIFYHEAD,
-	XP_TE_NOTIFYNUMBER,		XP_TE_NOTIFYOFFLINE,
-	XP_TE_NOTIFYONLINE,		XP_TE_PART,
-	XP_TE_PARTREASON,		XP_TE_PINGREP,
-	XP_TE_PINGTIMEOUT,		XP_TE_PRIVMSG,
-	XP_TE_DPRIVMSG,		XP_TE_ALREADYPROCESS,
-	XP_TE_QUIT,		XP_TE_RAWMODES,
-	XP_TE_WALLOPS,		XP_TE_RESOLVINGUSER,
-	XP_TE_SERVERCONNECTED,		XP_TE_SERVERERROR,
-	XP_TE_SERVERLOOKUP,		XP_TE_SERVNOTICE,
-	XP_TE_SERVTEXT,		XP_TE_STOPCONNECT,
-	XP_TE_TOPIC,		XP_TE_TOPICDATE,
-	XP_TE_NEWTOPIC,		XP_TE_UKNHOST,
-	XP_TE_USERLIMIT,		XP_TE_USERSONCHAN,
-	XP_TE_WHOIS5,		XP_TE_WHOIS2,
-	XP_TE_WHOIS6,		XP_TE_WHOIS_ID,
-	XP_TE_WHOIS_AUTH,	XP_TE_WHOIS_REALHOST,
-	XP_TE_WHOIS4,		XP_TE_WHOIS4T,
-	XP_TE_WHOIS1,		XP_TE_WHOIS3,
-	XP_TE_UJOIN,		XP_TE_UPART,
-	XP_TE_UPARTREASON,		XP_TE_UKICK,
-	XP_TE_UINVITE,		XP_TE_UCHANMSG,
-	XP_TE_UCHANGENICK,
+	XP_TE_FOUNDIP,		XP_TE_GENMSG,
+	XP_TE_IGNOREADD,		XP_TE_IGNORECHANGE,
+	XP_TE_IGNOREFOOTER,		XP_TE_IGNOREHEADER,
+	XP_TE_IGNOREREMOVE,		XP_TE_IGNOREEMPTY,
+	XP_TE_INVITE,		XP_TE_INVITED,
+	XP_TE_JOIN,		XP_TE_KEYWORD,
+	XP_TE_KICK,		XP_TE_KILL,
+	XP_TE_MSGSEND,		XP_TE_MOTD,
+	XP_TE_MOTDSKIP,		XP_TE_NICKCLASH,
+	XP_TE_NICKFAIL,		XP_TE_NODCC,
+	XP_TE_NOCHILD,		XP_TE_NOTICE,
+	XP_TE_NOTICESEND,		XP_TE_NOTIFYEMPTY,
+	XP_TE_NOTIFYHEAD,		XP_TE_NOTIFYNUMBER,
+	XP_TE_NOTIFYOFFLINE,		XP_TE_NOTIFYONLINE,
+	XP_TE_PART,		XP_TE_PARTREASON,
+	XP_TE_PINGREP,		XP_TE_PINGTIMEOUT,
+	XP_TE_PRIVMSG,		XP_TE_DPRIVMSG,
+	XP_TE_ALREADYPROCESS,		XP_TE_QUIT,
+	XP_TE_RAWMODES,		XP_TE_WALLOPS,
+	XP_TE_RESOLVINGUSER,		XP_TE_SERVERCONNECTED,
+	XP_TE_SERVERERROR,		XP_TE_SERVERLOOKUP,
+	XP_TE_SERVNOTICE,		XP_TE_SERVTEXT,
+	XP_TE_STOPCONNECT,		XP_TE_TOPIC,
+	XP_TE_TOPICDATE,		XP_TE_NEWTOPIC,
+	XP_TE_UKNHOST,		XP_TE_USERLIMIT,
+	XP_TE_USERSONCHAN,		XP_TE_WHOIS5,
+	XP_TE_WHOIS2,		XP_TE_WHOIS6,
+	XP_TE_WHOIS_ID,		XP_TE_WHOIS_AUTH,
+	XP_TE_WHOIS_REALHOST,		XP_TE_WHOIS4,
+	XP_TE_WHOIS4T,		XP_TE_WHOIS1,
+	XP_TE_WHOIS3,		XP_TE_UJOIN,
+	XP_TE_UPART,		XP_TE_UPARTREASON,
+	XP_TE_UKICK,		XP_TE_UINVITE,
+	XP_TE_UCHANMSG,		XP_TE_UCHANGENICK,
 	NUM_XP
 };
