@@ -290,7 +290,9 @@ errorstring (int err)
 		return "";
 	case 0:
 		return _("Remote host closed socket");
-#ifdef WIN32
+#ifndef WIN32
+	}
+#else
 	case WSAECONNREFUSED:
 		return _("Connection refused");
 	case WSAENETUNREACH:
@@ -302,17 +304,53 @@ errorstring (int err)
 		return _("Cannot assign that address");
 	case WSAECONNRESET:
 		return _("Connection reset by peer");
-	default:
-		{
-			static char tbuf[16];
-			sprintf (tbuf, "(%d)", err);
-			return tbuf;
-		}
-#else
-	default:
-		return strerror (err);
-#endif
 	}
+
+	/* can't use strerror() on Winsock errors! */
+	if (err >= WSABASEERR)
+	{
+		static char tbuf[384];
+		OSVERSIONINFO osvi;
+
+		osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
+		GetVersionEx (&osvi);
+
+		/* FormatMessage works on WSA*** errors starting from Win2000 */
+		if (osvi.dwMajorVersion >= 5)
+		{
+			if (FormatMessageA (FORMAT_MESSAGE_FROM_SYSTEM |
+									  FORMAT_MESSAGE_IGNORE_INSERTS |
+									  FORMAT_MESSAGE_MAX_WIDTH_MASK,
+									  NULL, err,
+									  MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
+									  tbuf, sizeof (tbuf), NULL))
+			{
+				int len;
+				char *utf;
+
+				tbuf[sizeof (tbuf) - 1] = 0;
+				len = strlen (tbuf);
+				if (len >= 2)
+					tbuf[len - 2] = 0;	/* remove the cr-lf */
+
+				/* now convert to utf8 */
+				utf = g_locale_to_utf8 (tbuf, -1, 0, 0, 0);
+				if (utf)
+				{
+					safe_strcpy (tbuf, utf, sizeof (tbuf));
+					g_free (utf);
+					return tbuf;
+				}
+			}
+		}	/* ! if (osvi.dwMajorVersion >= 5) */
+
+		/* fallback to error number */
+		sprintf (tbuf, "%s %d", _("Error"), err);
+		return tbuf;
+	} /* ! if (err >= WSABASEERR) */
+#endif	/* ! WIN32 */
+
+	return strerror (err);
 }
 
 int
@@ -420,7 +458,7 @@ strip_color (char *text)
 #if defined (USING_LINUX) || defined (USING_FREEBSD)
 
 static void
-get_cpu_info (int *mhz, int *cpus)
+get_cpu_info (double *mhz, int *cpus)
 {
 
 #ifdef USING_LINUX
@@ -467,6 +505,7 @@ get_cpu_info (int *mhz, int *cpus)
 	u_long freq;
 	size_t len;
 
+	freq = 0;
 	*mhz = 0;
 	*cpus = 0;
 
@@ -513,7 +552,7 @@ get_cpu_str (void)
 	static char verbuf[64];
 	OSVERSIONINFO osvi;
 	SYSTEM_INFO si;
-	int mhz;
+	double mhz;
 
 	osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
 	GetVersionEx (&osvi);
@@ -521,9 +560,13 @@ get_cpu_str (void)
 
 	mhz = get_mhz ();
 	if (mhz)
-		sprintf (verbuf, "Windows %ld.%ld [i%d86/%dMHz]",
-			osvi.dwMajorVersion, osvi.dwMinorVersion, si.wProcessorLevel, mhz);
-	else
+	{
+		double cpuspeed = ( mhz > 1000 ) ? mhz / 1000 : mhz;
+		const char *cpuspeedstr = ( mhz > 1000 ) ? "GHz" : "MHz";
+		sprintf (verbuf, "Windows %ld.%ld [i%d86/%.2f%s]",
+					osvi.dwMajorVersion, osvi.dwMinorVersion, si.wProcessorLevel, 
+					cpuspeed, cpuspeedstr);
+	} else
 		sprintf (verbuf, "Windows %ld.%ld [i%d86]",
 			osvi.dwMajorVersion, osvi.dwMinorVersion, si.wProcessorLevel);
 
@@ -536,7 +579,8 @@ char *
 get_cpu_str (void)
 {
 #if defined (USING_LINUX) || defined (USING_FREEBSD)
-	int mhz, cpus;
+	double mhz;
+	int cpus = 1;
 #endif
 	struct utsname un;
 	static char *buf = NULL;
@@ -551,12 +595,18 @@ get_cpu_str (void)
 #if defined (USING_LINUX) || defined (USING_FREEBSD)
 	get_cpu_info (&mhz, &cpus);
 	if (mhz)
+	{
+		double cpuspeed = ( mhz > 1000 ) ? mhz / 1000 : mhz;
+		const char *cpuspeedstr = ( mhz > 1000 ) ? "GHz" : "MHz";
 		snprintf (buf, 128,
-					 (cpus == 1) ? "%s %s [%s/%dMHz]" : "%s %s [%s/%dMHz/SMP]",
-					 un.sysname, un.release, un.machine, mhz);
-	else
+					(cpus == 1) ? "%s %s [%s/%.2f%s]" : "%s %s [%s/%.2f%s/SMP]",
+					un.sysname, un.release, un.machine,
+					cpuspeed, cpuspeedstr);
+	} else
 #endif
-		snprintf (buf, 128, "%s %s [%s]", un.sysname, un.release, un.machine);
+		snprintf (buf, 128,
+					(cpus == 1) ? "%s %s [%s]" : "%s %s [%s/SMP]",
+					un.sysname, un.release, un.machine);
 
 	return buf;
 }
@@ -1150,7 +1200,7 @@ make_ping_time (void)
  */
 
 int
-rfc_casecmp (char *s1, char *s2)
+rfc_casecmp (const char *s1, const char *s2)
 {
 	register unsigned char *str1 = (unsigned char *) s1;
 	register unsigned char *str2 = (unsigned char *) s2;
