@@ -65,6 +65,7 @@ GSList *dcc_list = 0;
 GSList *ignore_list = 0;
 GSList *usermenu_list = 0;
 GSList *urlhandler_list = 0;
+GSList *tabmenu_list = 0;
 static GSList *away_list = 0;
 
 static int in_xchat_exit = FALSE;
@@ -353,11 +354,13 @@ set_server_defaults (server *serv)
 	serv->is_away = FALSE;
 	serv->supports_watch = FALSE;
 	serv->bad_prefix = FALSE;
+	serv->use_who = TRUE;
 }
 
 static server *
 new_server (void)
 {
+	static int id = 0;
 	server *serv;
 
 	serv = malloc (sizeof (struct server));
@@ -366,6 +369,7 @@ new_server (void)
 	/* use server.c and proto-irc.c functions */
 	server_fill_her_up (serv);
 
+	serv->id = id++;
 	serv->sok = -1;
 	strcpy (serv->nick, prefs.nick1);
 	set_server_defaults (serv);
@@ -714,6 +718,11 @@ find_away_message (struct server *serv, char *nick)
 	"NAME Connect as IRC server\n"		"CMD newserver %s\n\n"
 #else
 #define defaultconf_urlhandlers \
+	"NAME SUB\n"								"CMD Epiphany...\n\n"\
+		"NAME Open\n"							"CMD !epiphany '%s'\n\n"\
+		"NAME Open in new tab\n"			"CMD !epiphany -n '%s'\n\n"\
+		"NAME Open in new window\n"		"CMD !epiphany -w '%s'\n\n"\
+	"NAME ENDSUB\n"							"CMD \n\n"\
 	"NAME SUB\n"								"CMD Netscape...\n\n"\
 		"NAME Open in existing\n"			"CMD !netscape -remote 'openURL(%s)'\n\n"\
 		"NAME Open in new window\n"		"CMD !netscape -remote 'openURL(%s,new-window)'\n\n"\
@@ -725,11 +734,11 @@ find_away_message (struct server *serv, char *nick)
 		"NAME Open in new tab\n"			"CMD !mozilla -remote 'openURL(%s,new-tab)'\n\n"\
 		"NAME Run new Mozilla\n"			"CMD !mozilla %s\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
-	"NAME SUB\n"								"CMD Mozilla Firebird...\n\n"\
-		"NAME Open in existing\n"			"CMD !MozillaFirebird -remote 'openURL(%s)'\n\n"\
-		"NAME Open in new window\n"		"CMD !MozillaFirebird -remote 'openURL(%s,new-window)'\n\n"\
-		"NAME Open in new tab\n"			"CMD !MozillaFirebird -remote 'openURL(%s,new-tab)'\n\n"\
-		"NAME Run new Mozilla Firebird\n"	"CMD !MozillaFirebird %s\n\n"\
+	"NAME SUB\n"								"CMD Mozilla FireFox...\n\n"\
+		"NAME Open in existing\n"			"CMD !firefox -remote 'openURL(%s)'\n\n"\
+		"NAME Open in new window\n"		"CMD !firefox -remote 'openURL(%s,new-window)'\n\n"\
+		"NAME Open in new tab\n"			"CMD !firefox -remote 'openURL(%s,new-tab)'\n\n"\
+		"NAME Run new Mozilla FireFox\n"	"CMD !firefox %s\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
 	"NAME SUB\n"								"CMD Galeon...\n\n"\
 		"NAME Open in existing\n"			"CMD !galeon -x '%s'\n\n"\
@@ -740,6 +749,7 @@ find_away_message (struct server *serv, char *nick)
 	"NAME SUB\n"								"CMD Opera...\n\n"\
 		"NAME Open in existing\n"			"CMD !opera -remote 'openURL(%s)'\n\n"\
 		"NAME Open in new window\n"		"CMD !opera -remote 'openURL(%s,new-window)'\n\n"\
+		"NAME Open in new tab\n"			"CMD !opera -remote 'openURL(%s,new-page)'\n\n"\
 		"NAME Run new Opera\n"				"CMD !opera %s\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
 	"NAME SUB\n"								"CMD Send URL to...\n\n"\
@@ -756,7 +766,7 @@ find_away_message (struct server *serv, char *nick)
 	"NAME Connect as IRC server\n"		"CMD newserver %s\n\n"
 #endif
 
-#ifndef WIN32
+#ifdef USE_SIGACTION
 /* Close and open log files on SIGUSR1. Usefull for log rotating */
 
 static void 
@@ -812,9 +822,10 @@ xchat_init (void)
 	}
 #else
 	WSAStartup(0x0101, &wsadata);
-#endif
+#endif	/* !USE_IPV6 */
+#endif	/* !WIN32 */
 
-#else
+#ifdef USE_SIGACTION
 	struct sigaction act;
 
 	/* ignore SIGPIPE's */
@@ -833,6 +844,11 @@ xchat_init (void)
 	act.sa_flags = 0;
 	sigemptyset (&act.sa_mask);
 	sigaction (SIGUSR2, &act, NULL);
+#else
+#ifndef WIN32
+	/* good enough for these old systems */
+	signal (SIGPIPE, SIG_IGN);
+#endif
 #endif
 
 	if (g_get_charset (&cs))
@@ -983,15 +999,16 @@ xchat_init (void)
 		"NAME %s\n"				"CMD whois %%s %%s\n\n"
 		"NAME %s\n"				"CMD dcc send %%s\n\n"
 		"NAME %s\n"				"CMD dcc chat %%s\n\n"
-		"NAME %s\n"				"CMD ping %%s\n\n"
-		"NAME %s\n"				"CMD clear\n\n",
+		"NAME %s\n"				"CMD clear\n\n"
+		"NAME %s\n"				"CMD ping %%s\n\n",
 				_("WhoIs"),
 				_("Send"),
 				_("Chat"),
-				_("Ping"),
-				_("Clear"));
+				_("Clear"),
+				_("Ping"));
 	list_loadconf ("dlgbuttons.conf", &dlgbutton_list, buf);
 
+	list_loadconf ("tabmenu.conf", &tabmenu_list, NULL);
 	list_loadconf ("ctcpreply.conf", &ctcp_list, defaultconf_ctcp);
 	list_loadconf ("commands.conf", &command_list, defaultconf_commands);
 	list_loadconf ("replace.conf", &replace_list, defaultconf_replace);
@@ -1042,10 +1059,6 @@ xchat_exit (void)
 	ignore_save ();
 	free_sessions ();
 	fe_exit ();
-
-#ifdef WIN32
-	WSACleanup ();
-#endif
 }
 
 #ifndef WIN32
@@ -1104,5 +1117,9 @@ main (int argc, char *argv[])
 	xchat_mem_list ();
 #endif
 
+#ifdef WIN32
+	WSACleanup ();
+#endif
+
 	return 0;
 }
