@@ -1,3 +1,6 @@
+#ifndef XCHAT_C_H
+#define XCHAT_C_H
+
 extern struct xchatprefs prefs;
 
 extern int auto_connect;
@@ -20,6 +23,7 @@ extern GSList *dcc_list;
 extern GSList *ignore_list;
 extern GSList *usermenu_list;
 extern GSList *urlhandler_list;
+extern GSList *tabmenu_list;
 
 session * find_channel (server *serv, char *chan);
 session * find_dialog (server *serv, char *nick);
@@ -33,3 +37,5 @@ void lag_check (void);
 void kill_session_callback (session * killsess);
 void xchat_exit (void);
 void xchat_exec (char *cmd);
+
+#endif
