@@ -31,6 +31,7 @@
 #include <gtk/gtkbox.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtktogglebutton.h>
+#include <gtk/gtkversion.h>
 
 #include "../common/xchat.h"
 #include "../common/fe.h"
@@ -164,6 +165,25 @@ fe_args (int argc, char *argv[])
 		}
 	}
 
+#ifdef WIN32
+	/* this is mainly for irc:// URL handling. When windows calls us from */
+	/* I.E, it doesn't give an option of "Start in" directory, like short */
+	/* cuts can. So we have to set the current dir manually, to the path  */
+	/* of the exe. */
+	{
+		char *tmp = strdup (argv[0]);
+		char *sl;
+
+		sl = strrchr (tmp, '\\');
+		if (sl)
+		{
+			*sl = 0;
+			chdir (tmp);
+		}
+		free (tmp);
+	}
+#endif
+
 	if (argc > 2 + offset)
 	{
 		if (!strcasecmp (argv[1+offset], "-d") || !strcasecmp (argv[1+offset], "--cfgdir"))
@@ -288,6 +308,9 @@ log_handler (const gchar   *log_domain,
 {
 	session *sess;
 
+	if (getenv ("XCHAT_WARNING_IGNORE"))
+		return;
+
 	sess = find_dialog (serv_list->data, "(warnings)");
 	if (!sess)
 		sess = new_ircwindow (serv_list->data, "(warnings)", SESS_DIALOG);
@@ -317,7 +340,7 @@ fe_new_window (session *sess)
 	mg_changui_new (sess, NULL, tab);
 
 #ifdef WIN32
-	g_log_set_handler ("Glib", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING, (GLogFunc)log_handler, 0);
+	g_log_set_handler ("GLib", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING, (GLogFunc)log_handler, 0);
 	g_log_set_handler ("GLib-GObject", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING, (GLogFunc)log_handler, 0);
 	g_log_set_handler ("Gdk", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING, (GLogFunc)log_handler, 0);
 	g_log_set_handler ("Gtk", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING, (GLogFunc)log_handler, 0);
@@ -407,55 +430,11 @@ fe_set_topic (session *sess, char *topic)
 	}
 }
 
-#ifdef WIN32
-#define WINVER 0x0501	/* needed for vc6? */
-#include <windows.h>
-#include <gdk/gdkwin32.h>
-
-/* Flash the taskbar button on Windows when there's a highlight event. */
-
-static void
-flash_window (GtkWidget *win)
-{
-	FLASHWINFO fi;
-	static HMODULE user = NULL;
-	static BOOL (*flash) (PFLASHWINFO) = NULL;
-
-	if (!user)
-	{
-		user = GetModuleHandleA ("USER32");
-		if (!user)
-			return;	/* this should never fail */
-	}
-
-	if (!flash)
-	{
-		flash = (void *)GetProcAddress (user, "FlashWindowEx");
-		if (!flash)
-			return;	/* this fails on NT4.0 and Win95 */
-	}
-
-	fi.cbSize = sizeof (fi);
-	fi.hwnd = GDK_WINDOW_HWND (win->window);
-	fi.dwFlags = FLASHW_ALL | FLASHW_TIMERNOFG;
-	fi.uCount = 0;
-	fi.dwTimeout = 500;
-	flash (&fi);
-	/*FlashWindowEx (&fi);*/
-}
-#endif
-
 void
 fe_set_hilight (struct session *sess)
 {
 	if (sess->gui->is_tab)
-	{
-		sess->nick_said = TRUE;
-		tab_set_attrlist (sess->res->tab, nickseen_list);
-#ifdef WIN32
-		flash_window (sess->gui->window);
-#endif
-	}
+		fe_set_tab_color (sess, 3, TRUE);	/* blue, with taskbar flash */
 }
 
 static void
@@ -580,9 +559,9 @@ fe_print_text (struct session *sess, char *text)
 	{
 		sess->new_data = TRUE;
 		if (sess->msg_said)
-			tab_set_attrlist (sess->res->tab, newmsg_list);
+			fe_set_tab_color (sess, 2, FALSE);
 		else
-			tab_set_attrlist (sess->res->tab, newdata_list);
+			fe_set_tab_color (sess, 1, FALSE);
 	}
 }
 
@@ -709,15 +688,49 @@ fe_set_throttle (server *serv)
 }
 
 void
-fe_ctrl_gui (session *sess, int action)
+fe_ctrl_gui (session *sess, int action, int arg)
 {
 	switch (action)
 	{
 	case 0:
 		gtk_widget_hide (sess->gui->window); break;
 	case 1:
-		gtk_widget_show (sess->gui->window); break;
+		gtk_widget_show (sess->gui->window);
+		gtk_window_present (GTK_WINDOW (sess->gui->window));
+		break;
 	case 2:
 		mg_bring_tofront (sess->res->tab); break;
+	case 3:
+		fe_set_tab_color (sess, -1, TRUE); break; /* flash */
+	case 4:
+		fe_set_tab_color (sess, arg, FALSE); break;
+	case 5:
+		gtk_window_iconify (GTK_WINDOW (sess->gui->window)); break;
 	}
 }
+
+void
+fe_confirm (const char *message, void (*yesproc)(void *), void (*noproc)(void *), void *ud)
+{
+}
+
+int
+fe_gui_info (session *sess, int info_type)
+{
+	switch (info_type)
+	{
+	case 0:	/* window status */
+		if (!GTK_WIDGET_VISIBLE (GTK_WINDOW (sess->gui->window)))
+			return 2;	/* hidden (iconified or systray) */
+#if GTK_CHECK_VERSION(2,4,0)
+		if (gtk_window_is_active (GTK_WINDOW (sess->gui->window)))
+#else
+		if (GTK_WINDOW (sess->gui->window)->is_active)
+#endif
+			return 1;	/* active/focused */
+
+		return 0;		/* normal (no keyboard focus or behind a window) */
+	}
+
+	return -1;
+}
