@@ -202,7 +202,9 @@ key_get_key_name (int keyval)
 
 /* Ok, here are the NOTES
 
-   key_handle_key_press now handles all the key presses and history_keypress is now defunct. It goes thru the linked list keys_root and finds a matching key. It runs the action func and switches on these values:
+   key_handle_key_press now handles all the key presses and history_keypress is
+   now defunct. It goes thru the linked list keys_root and finds a matching
+   key. It runs the action func and switches on these values:
    0) Return
    1) Find next
    2) stop signal and return
@@ -236,7 +238,7 @@ key_get_key_name (int keyval)
  */
 
 gboolean
-key_handle_key_pressAFTER (GtkWidget *wid, GdkEventKey *evt, session *sess)
+key_handle_key_press (GtkWidget *wid, GdkEventKey *evt, session *sess)
 {
 	struct key_binding *kb, *last = NULL;
 	int keyval = evt->keyval;
@@ -305,16 +307,6 @@ key_handle_key_pressAFTER (GtkWidget *wid, GdkEventKey *evt, session *sess)
 	return 0;
 }
 
-gboolean
-key_handle_key_press (GtkWidget *wid, GdkEventKey *evt, session *sess)
-{
-	if (evt->keyval == GDK_space || evt->keyval == GDK_Return ||
-		 evt->keyval == GDK_KP_Enter)
-		return key_handle_key_pressAFTER (wid, evt, sess);
-
-	return FALSE;
-}
-
 /* Walks keys_root and free()'s everything */
 /*static void
 key_free_all ()
@@ -387,9 +379,9 @@ key_load_defaults ()
 		"A\n3\nChange Page\nD1:3\nD2!\n\n"\
 		"A\n2\nChange Page\nD1:2\nD2!\n\n"\
 		"A\n1\nChange Page\nD1:1\nD2!\n\n"\
-		"C\no\nInsert in Buffer\nD1:%O\nD2!\n\n"\
-		"C\nb\nInsert in Buffer\nD1:%B\nD2!\n\n"\
-		"C\nk\nInsert in Buffer\nD1:%C\nD2!\n\n"\
+		"C\no\nInsert in Buffer\nD1:\nD2!\n\n"\
+		"C\nb\nInsert in Buffer\nD1:\nD2!\n\n"\
+		"C\nk\nInsert in Buffer\nD1:\nD2!\n\n"\
 		"S\nNext\nChange Selected Nick\nD1!\nD2!\n\n"\
 		"S\nPrior\nChange Selected Nick\nD1:Up\nD2!\n\n"\
 		"None\nNext\nScroll Page\nD1!\nD2!\n\n"\
@@ -1334,7 +1326,7 @@ key_action_tab_comp (GtkWidget * wid, GdkEventKey * ent, char *d1, char *d2,
 		}
 
 		/* check if we can match a command */
-		else if (chan_start == 0 && text[0] == '/')
+		else if (chan_start == 0 && text[0] == prefs.cmdchar[0])
 		{
 			tab_comp_cmd (sess, wid);
 			return 2;
@@ -1706,18 +1698,18 @@ tab_comp_cmd (session *sess, GtkWidget * t)
 		if (strlen (last) > (sizeof (buf) - 1))
 			return;
 		if (postfix == NULL)
-			snprintf (buf, sizeof (buf), "/%s ", last);
+			snprintf (buf, sizeof (buf), "%c%s ", prefs.cmdchar[0], last);
 		else
-			snprintf (buf, sizeof (buf), "/%s %s", last, postfix);
+			snprintf (buf, sizeof (buf), "%c%s %s", prefs.cmdchar[0], last, postfix);
 		gtk_entry_set_text (GTK_ENTRY (t), buf);
 		gtk_editable_set_position (GTK_EDITABLE (t), -1);
 		return;
 	} else if (strlen (lcmd) > (sizeof (buf) - 1))
 		return;
 	if (postfix == NULL)
-		snprintf (buf, sizeof (buf), "/%s", lcmd);
+		snprintf (buf, sizeof (buf), "%c%s", prefs.cmdchar[0], lcmd);
 	else
-		snprintf (buf, sizeof (buf), "/%s %s", lcmd, postfix);
+		snprintf (buf, sizeof (buf), "%c%s %s", prefs.cmdchar[0], lcmd, postfix);
 	gtk_entry_set_text (GTK_ENTRY (t), buf);
 	gtk_editable_set_position (GTK_EDITABLE (t), -1);
 }
@@ -2168,6 +2160,14 @@ tab_nick_comp (session *sess, GtkWidget *t, int shift)
 		}
 	}
 
+	{
+		/* This fixes invalid UTF-8 sequence by completion of
+		   a multibyte nick. -- FIXME: only a bandaid solution -- */
+		char *valid_end;
+		g_utf8_validate (buf, -1, (const gchar **)&valid_end);
+		(*valid_end) = 0;
+	}
+
 	gtk_entry_set_text (GTK_ENTRY (t), buf);
 	/* GeEkMaN: Restore the last cursor position */
 	gtk_editable_set_position (GTK_EDITABLE (t), cursor_pos);
