@@ -112,6 +112,86 @@ mg_create_tab_colors (void)
 		plain_list = pango_attr_list_new ();
 }
 
+#ifdef WIN32
+#define WINVER 0x0501	/* needed for vc6? */
+#include <windows.h>
+#include <gdk/gdkwin32.h>
+
+/* Flash the taskbar button on Windows when there's a highlight event. */
+
+static void
+flash_window (GtkWidget *win)
+{
+	FLASHWINFO fi;
+	static HMODULE user = NULL;
+	static BOOL (*flash) (PFLASHWINFO) = NULL;
+
+	if (!user)
+	{
+		user = GetModuleHandleA ("USER32");
+		if (!user)
+			return;	/* this should never fail */
+	}
+
+	if (!flash)
+	{
+		flash = (void *)GetProcAddress (user, "FlashWindowEx");
+		if (!flash)
+			return;	/* this fails on NT4.0 and Win95 */
+	}
+
+	fi.cbSize = sizeof (fi);
+	fi.hwnd = GDK_WINDOW_HWND (win->window);
+	fi.dwFlags = FLASHW_ALL | FLASHW_TIMERNOFG;
+	fi.uCount = 0;
+	fi.dwTimeout = 500;
+	flash (&fi);
+	/*FlashWindowEx (&fi);*/
+}
+#endif
+
+/* set a tab plain, red, light-red, or blue */
+
+void
+fe_set_tab_color (struct session *sess, int col, int flash)
+{
+	if (sess->gui->is_tab && (col == 0 || sess != current_tab))
+	{
+		switch (col)
+		{
+		case 0:	/* no particular color (theme default) */
+			sess->new_data = FALSE;
+			sess->msg_said = FALSE;
+			sess->nick_said = FALSE;
+			tab_set_attrlist (sess->res->tab, plain_list);
+			break;
+		case 1:	/* new data has been displayed (dark red) */
+			sess->new_data = TRUE;
+			sess->msg_said = FALSE;
+			sess->nick_said = FALSE;
+			tab_set_attrlist (sess->res->tab, newdata_list);
+			break;
+		case 2:	/* new message arrived in channel (light red) */
+			sess->new_data = FALSE;
+			sess->msg_said = TRUE;
+			sess->nick_said = FALSE;
+			tab_set_attrlist (sess->res->tab, newmsg_list);
+			break;
+		case 3:	/* your nick has been seen (blue) */
+			sess->new_data = FALSE;
+			sess->msg_said = FALSE;
+			sess->nick_said = TRUE;
+			tab_set_attrlist (sess->res->tab, nickseen_list);
+			break;
+		}
+	}
+
+#ifdef WIN32
+	if (flash)
+		flash_window (sess->gui->window);
+#endif
+}
+
 /* change the little icon to the left of your nickname */
 
 void
@@ -218,8 +298,8 @@ fe_set_title (session *sess)
 	switch (type)
 	{
 	case SESS_DIALOG:
-		snprintf (tbuf, sizeof (tbuf), "X-Chat ["VERSION"]: Dialog with %s @ %s",
-					 sess->channel, sess->server->servername);
+		snprintf (tbuf, sizeof (tbuf), "X-Chat ["VERSION"]: %s %s @ %s",
+					 _("Dialog with"), sess->channel, sess->server->servername);
 		break;
 	case SESS_SERVER:
 		snprintf (tbuf, sizeof (tbuf), "X-Chat ["VERSION"]: %s @ %s",
@@ -342,7 +422,7 @@ mg_focus (session *sess)
 		sess->nick_said = FALSE;
 		sess->msg_said = FALSE;
 		sess->new_data = FALSE;
-		tab_set_attrlist (sess->res->tab, plain_list);
+		fe_set_tab_color (sess, 0, FALSE);
 	}
 }
 
@@ -481,7 +561,11 @@ mg_set_topic_tip (session *sess)
 			add_tip (sess->gui->topic_entry, _("No topic is set"));
 		break;
 	default:
-		add_tip (sess->gui->topic_entry, GTK_ENTRY (sess->gui->topic_entry)->text);
+		if (GTK_ENTRY (sess->gui->topic_entry)->text &&
+			 GTK_ENTRY (sess->gui->topic_entry)->text[0])
+			add_tip (sess->gui->topic_entry, GTK_ENTRY (sess->gui->topic_entry)->text);
+		else
+			add_tip (sess->gui->topic_entry, ""); /* hSP: set topic to empty in case it isn't set.. */
 	}
 }
 
@@ -909,7 +993,7 @@ mg_color_insert (GtkWidget *item, gpointer userdata)
 {
 	char buf[32];
 
-	sprintf (buf, "%%C%02d", GPOINTER_TO_INT (userdata));
+	sprintf (buf, "\003%02d", GPOINTER_TO_INT (userdata));
 	key_action_insert (current_sess->gui->input_box, 0, buf, 0, 0);
 }
 
@@ -1004,6 +1088,9 @@ mg_tab_press_cb (GtkWidget *wid, GdkEventButton *event, session *sess)
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
 	gtk_widget_show (item);
 
+	if (sess && tabmenu_list)
+		menu_create (menu, tabmenu_list, sess->channel, FALSE);
+
 	g_signal_connect (G_OBJECT (menu), "selection-done",
 							G_CALLBACK (mg_menu_destroy), menu);
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, event->time);
@@ -1660,7 +1747,7 @@ mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even,
 			word+1 : word, FALSE);
 		break;
 	case WORD_CHANNEL:
-		if (*word == '@' || *word == '+')
+		if (*word == '@' || *word == '+' || *word=='^' || *word=='%' || *word=='*')
 			word++;
 		menu_chanmenu (sess, even, word);
 		break;
@@ -1701,6 +1788,22 @@ mg_update_xtext (GtkWidget *wid)
 	gtk_xtext_refresh (xtext, FALSE);
 }
 
+/* handle errors reported by xtext */
+
+static void
+mg_xtext_error (int type)
+{
+	switch (type)
+	{
+	case 0:
+		gtkutil_simpledialog (_("Unable to set transparent background!\n\n"
+										"You may be using a non-compliant window\n"
+										"manager that is not currently supported.\n"));
+		prefs.transparent = 0;
+		/* no others exist yet */
+	}
+}
+
 static void
 mg_create_textarea (session_gui *gui, GtkWidget *box)
 {
@@ -1721,7 +1824,7 @@ mg_create_textarea (session_gui *gui, GtkWidget *box)
 	xtext = GTK_XTEXT (gui->xtext);
 	gtk_xtext_set_max_indent (xtext, prefs.max_auto_indent);
 	gtk_xtext_set_thin_separator (xtext, prefs.thin_separator);
-	gtk_xtext_set_error_function (xtext, gtkutil_simpledialog);
+	gtk_xtext_set_error_function (xtext, mg_xtext_error);
 	gtk_xtext_set_urlcheck_function (xtext, mg_word_check);
 	gtk_xtext_set_max_lines (xtext, prefs.max_lines);
 	gtk_container_add (GTK_CONTAINER (frame), GTK_WIDGET (xtext));
@@ -1811,7 +1914,10 @@ mg_create_userlist (session_gui *gui, GtkWidget *box, int pack)
 	gtk_widget_set_size_request (ulist, 86, -1);
 
 	if (prefs.style_namelistgad)
+	{
 		gtk_widget_set_style (ulist, input_style);
+		gtk_widget_modify_base (ulist, GTK_STATE_NORMAL, &colors[19]);
+	}
 
 	mg_create_meters (gui, vbox);
 
@@ -1968,8 +2074,6 @@ mg_create_entry (session *sess, GtkWidget *box)
 	gui->input_box = entry = gtk_entry_new ();
 	gtk_widget_set_name (entry, "xchat-inputbox");
 	gtk_entry_set_max_length (GTK_ENTRY (gui->input_box), 2048);
-	g_signal_connect_after (G_OBJECT (entry), "key_press_event",
-							G_CALLBACK (key_handle_key_pressAFTER), NULL);
 	g_signal_connect (G_OBJECT (entry), "key_press_event",
 							G_CALLBACK (key_handle_key_press), NULL);
 	g_signal_connect (G_OBJECT (entry), "focus_in_event",
@@ -2012,7 +2116,7 @@ mg_create_tabs (session_gui *gui, GtkWidget *box)
 	if (prefs.tabs_position == 2 || prefs.tabs_position == 3)
 		vert = TRUE;
 
-	gui->tabs_box = tab_group_new (mg_switch_tab_cb, vert, FALSE);
+	gui->tabs_box = tab_group_new (mg_switch_tab_cb, vert, prefs.tab_sort);
 	gtk_table_attach (GTK_TABLE (gui->main_table), gui->tabs_box,
 						1, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 }
