@@ -107,6 +107,16 @@ goto_url (char *url)
 #else
 	char tbuf[512], *moz;
 
+	/* gnome 2.4+ has this */
+	moz = g_find_program_in_path ("gnome-open");
+	if (moz)
+	{
+		snprintf (tbuf, sizeof (tbuf), "%s %s", moz, url);
+		g_free (moz);
+		xchat_exec (tbuf);
+		return;
+	}
+
 	moz = g_find_program_in_path ("gnome-moz-remote");
 	if (moz)
 	{
@@ -387,7 +397,7 @@ is_in_path (char *cmd)
 
 /* append items to "menu" using the (struct popup*) list provided */
 
-static void
+void
 menu_create (GtkWidget *menu, GSList *list, char *target, int check_path)
 {
 	struct popup *pop;
@@ -628,6 +638,12 @@ open_url_cb (GtkWidget *item, char *url)
 }
 #endif
 
+static void
+copy_to_clipboard_cb (GtkWidget *item, char *url)
+{
+	gtkutil_copy_to_clipboard (item, NULL, url);
+}
+
 void
 menu_urlmenu (GdkEventButton *event, char *url)
 {
@@ -678,6 +694,7 @@ menu_urlmenu (GdkEventButton *event, char *url)
 	}
 	menu_quick_item (0, 0, menu, 1, 0);
 
+	menu_quick_item_with_callback (copy_to_clipboard_cb, _("Copy selected URL"), menu, str_copy);
 #ifdef WIN32
 	menu_quick_item_with_callback (open_url_cb, "Open URL", menu, str_copy);
 #endif
@@ -1201,7 +1218,7 @@ menu_saveexit (GtkWidget *wid, gpointer none)
 static void
 menu_docs (GtkWidget *wid, gpointer none)
 {
-	goto_url ("http://xchat.org/docs.html");
+	goto_url ("http://xchat.org/docs/");
 }
 
 /*static void
@@ -1339,6 +1356,19 @@ create_icon_menu (char *labeltext, void *stock_name, int is_stock)
 	return item;
 }
 
+#if GTK_CHECK_VERSION(2,4,0)
+
+/* Override the default GTK2.4 handler, which would make menu
+   bindings not work when the menu-bar is hidden. */
+static gboolean
+menu_canacaccel (GtkWidget *widget, guint signal_id, gpointer user_data)
+{
+	/* GTK2.2 behaviour */
+	return GTK_WIDGET_IS_SENSITIVE (widget);
+}
+
+#endif
+
 GtkWidget *
 menu_create_main (void *accel_group, int bar, int away, int toplevel,
 						GtkWidget **away_item_ret, GtkWidget **user_menu_ret)
@@ -1359,6 +1389,11 @@ menu_create_main (void *accel_group, int bar, int away, int toplevel,
 	else
 		menu_bar = gtk_menu_new ();
 
+#if GTK_CHECK_VERSION(2,4,0)
+	g_signal_connect (G_OBJECT (menu_bar), "can-activate-accel",
+							G_CALLBACK (menu_canacaccel), 0);
+#endif
+
 	mymenu[19-menuoffset].state = prefs.invisible;
 	mymenu[20-menuoffset].state = prefs.wallops;
 	mymenu[21-menuoffset].state = prefs.servernotice;
