@@ -273,6 +273,11 @@ servlist_addnet_cb (GtkWidget *item, GtkTreeView *treeview)
 
 	store = (GtkListStore *)gtk_tree_view_get_model (treeview);
 	net = servlist_net_add (_("New Network"), "");
+#ifdef WIN32
+	/* Windows gets UTF-8 for new users. Unix gets "System Default",
+		which is often UTF-8 anyway! */
+	net->encoding = strdup ("UTF-8");
+#endif
 	servlist_server_add (net, "newserver/6667");
 
 	gtk_list_store_append (store, &iter);
@@ -1063,6 +1068,7 @@ servlist_create_editbox (GtkWidget *box)
 	entry_real = servlist_create_entry (table, _("Real Name:"), 3,																	G_STRUCT_OFFSET (ircnet, real));
 	entry_pass = servlist_create_entry (table, _("Server Password:"), 4,
 											G_STRUCT_OFFSET (ircnet, pass));
+	gtk_entry_set_visibility (GTK_ENTRY (entry_pass), 0);
 	entry_join = servlist_create_entry (table, _("Join Channels:"), 5,
 											G_STRUCT_OFFSET (ircnet, autojoin));
 	add_tip (entry_join, _("Channels to join, separated by commas, but not spaces!"));
